#!/usr/bin/python
# -*- coding: utf-8 -*-
# Copyright 2020 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

#############################################
#                WARNING                    #
#############################################
#
# This file is auto generated by the resource
#   module builder playbook.
#
# Do not edit this file manually.
#
# Changes to this file will be over written
#   by the resource module builder.
#
# Changes should be made in the model used to
#   generate this file or in the resource module
#   builder template.
#
#############################################

"""
The module file for iosxr_ospfv3
"""

from __future__ import absolute_import, division, print_function


__metaclass__ = type

DOCUMENTATION = """
module: iosxr_ospfv3
version_added: 1.1.0
short_description: Resource module to configure OSPFv3.
description:
  - This module manages global ospfv3 configuration on devices running Cisco IOS-XR
author: Rohit Thakur (@rohitthakur2590)
notes:
  - This module works with connection C(network_cli). See L(the IOS-XR Platform Options,../network/user_guide/platform_iosxr.html)
options:
  running_config:
    description:
      - This option is used only with state I(parsed).
      - The value of this option should be the output received from the IOS-XR device
        by executing the command B(show running-config router ospfv3).
      - The state I(parsed) reads the configuration from C(running_config) option and
        transforms it into Ansible structured data as per the resource module's argspec
        and the value is then returned in the I(parsed) key within the result.
    type: str
  config:
    description: A list of ospfv3 process configuration
    type: dict
    suboptions:
      processes:
        description: A list of ospfv3 instances configuration
        type: list
        elements: dict
        suboptions:
          process_id:
            description: The OSPFv3 Process ID
            type: str
            required: true
          address_family_unicast:
            description: Enable unicast topology for ipv4 address family
            type: bool
          authentication:
            description: Enable authentication
            type: dict
            suboptions:
              disable:
                description: Do not authenticate OSPFv3 packets
                type: bool
                default: false
              ipsec:
                description: Specify IPSec AH authentication attributes
                type: dict
                suboptions:
                  spi:
                    description: Specify the Security Parameter Index value
                    type: int
                  algorithim_type:
                    description: Specify the type of algorithim
                    type: str
                    choices: ["md5", "sha1"]
                  key:
                    description: Specify key
                    type: str
                  clear_key:
                    description: Specify key in cleartext form
                    type: str
                  password_key:
                    description: Specify key in encrypted form
                    type: str
          auto_cost:
            description: Calculate ospfv3 interface cost according to bandwidth
            type: dict
            suboptions:
              reference_bandwidth:
                description: Specify reference bandwidth in megabits per sec
                type: int
              disable:
                description: Assign ospfv3 cost based on interface type
                type: bool
          bfd:
            description: Configure BFD parameters
            type: dict
            suboptions:
              fast_detect:
                description: Configure fast detection
                type: dict
                suboptions:
                  set:
                    description: Enable fast detection only
                    type: bool
                  strict_mode:
                    description: Hold down neighbor session until BFD session is up
                    type: bool
              minimum_interval:
                description: Hello interval in milli-seconds
                type: int
              multiplier:
                description: Detect multiplier
                type: int
          areas:
            description: Configure ospfv3 areas' properties
            type: list
            elements: dict
            suboptions:
              area_id:
                description: Area ID as IP address or integer
                type: str
                required: True
              authentication:
                description: Enable authentication
                type: dict
                suboptions:
                  disable:
                    description: Do not authenticate OSPFv3 packets
                    type: bool
                    default: false
                  ipsec:
                    description: Specify IPSec AH authentication attributes
                    type: dict
                    suboptions:
                      spi:
                        description: Specify the Security Parameter Index value
                        type: int
                      algorithim_type:
                        description: Specify the type of algorithim
                        type: str
                        choices: ["md5", "sha1"]
                      key:
                        description: Specify key
                        type: str
                      clear_key:
                        description: Specify key in cleartext form
                        type: str
                      password_key:
                        description: Specify key in encrypted form
                        type: str
              bfd:
                description: Configure BFD parameters
                type: dict
                suboptions:
                  fast_detect:
                    description: Configure fast detection
                    type: dict
                    suboptions:
                      set:
                        description: Enable fast detection only
                        type: bool
                      strict_mode:
                        description: Hold down neighbor session until BFD session is up
                        type: bool
                  minimum_interval:
                    description: Hello interval in milli-seconds
                    type: int
                  multiplier:
                    description: Detect multiplier
                    type: int
              cost:
                description: Interface cost
                type: int
              database_filter:
                description: Filter LSAs during synchronization and flooding
                type: dict
                suboptions:
                  all_outgoing_lsa:
                    description: Filter all outgoing LSA
                    type: bool
              dead_interval:
                description: Interval after which a neighbor is declared dead
                type: int
              default_cost:
                description: Set the summary default-cost of a NSSA/stub area. Stub's advertised external route metric
                type: int
              demand_circuit:
                description: Enable/Disable ospfv3 demand circuit
                type: bool
              distrinbute_rib_prefix_list_name:
                description: Filter LSAs during synchronization and flooding
                type: str
              fast_reroute:
                description: Specify IP Fast Reroute
                type: dict
                suboptions:
                  disabled:
                    description: Disable IP fast reroute
                    type: bool
                  per_link:
                    description: Specify per-prefix computation
                    type: dict
                    suboptions:
                      information_type:
                        description: Specify per-link LFA exclusion or FRR LFA candidate information
                        type: str
                        choices: ["exclude", "lfa_candidate"]
                      use_candidate_only:
                        description: Enable/Disable backup selection from candidate-list only
                        type: bool
                      interface:
                        description: Specify Per-link LFA exclusion information
                        type: dict
                        suboptions:
                          bvi:
                            description: Specify Bridge-Group Virtual Interface
                            type: list
                            elements: int
                          bundle_ether:
                            description: Specify Aggregated Ethernet interface(s)
                            type: list
                            elements: int
                          pos_int:
                            description: Specify Aggregated pos interface(s)
                            type: list
                            elements: int
                          fast_ethernet:
                            description: Specify FastEthernet/IEEE 802.3 interface(s)
                            type: list
                            elements: str
                          fiftygige:
                            description: Specify FiftyGigE/IEEE 802.3 interface(s)
                            type: list
                            elements: str
                          fortygige:
                            description: Specify FortyGigE/IEEE 802.3 interface(s)
                            type: list
                            elements: str
                          fourhundredgige:
                            description: Specify FourHundredGigE/IEEE 802.3 interface(s)
                            type: list
                            elements: str
                          gigabitethernet:
                            description: Specify GigabitEthernet/IEEE 802.3 interface(s)
                            type: list
                            elements: str
                          hundredgige:
                            description: Specify HundredGigE/IEEE 802.3 interface(s)
                            type: list
                            elements: str
                          mgmteth:
                            description: Specify MgmtEth/IEEE 802.3 interface(s)
                            type: list
                            elements: str
                          multilink:
                            description: Specify Multilink network interface(s)
                            type: list
                            elements: str
                          pw_ether:
                            description: Specify PWHE Ethernet Interface
                            type: list
                            elements: int
                          pw_iw:
                            description: Specify PWHE VC11 IP Interworking Interface
                            type: list
                            elements: int
                          srp:
                            description: Specify SRP interface(s)
                            type: list
                            elements: str
                          serial:
                            description: Specify Serial network interface(s)
                            type: list
                            elements: str
                          tengige:
                            description: Specify TenGigabitEthernet/IEEE 802.3 interface(s)
                            type: list
                            elements: str
                          twentyfivegige:
                            description: Specify TwentyFiveGigabitEthernet/IEEE 802.3 interface(s)
                            type: list
                            elements: str
                          twohundredgige:
                            description: Specify TwoHundredGigE/IEEE 802.3 interface(s)
                            type: list
                            elements: str
                          nve:
                            description: Specify Network Virtualization Endpoint Interface(s)
                            type: list
                            elements: int
                          tunnel_ip:
                            description: Specify GRE/IPinIP Tunnel Interface(s)
                            type: list
                            elements: int
                          tunnel_ipsec:
                            description: Specify IPSec Tunnel interface(s)
                            type: list
                            elements: int
                          tunnel_mte:
                            description: Specify MPLS Traffic Engineering P2MP Tunnel interface(s)
                            type: list
                            elements: int
                          tunnel_mpls:
                            description: MPLS Transport Protocol Tunnel interface
                            type: int
                  per_prefix:
                    description: Specify per-prefix computation
                    type: dict
                    suboptions:
                      information_type:
                        description: Specify per_prefix LFA exclusion or FRR LFA candidate information
                        type: str
                        choices: ["exclude", "lfa_candidate"]
                      use_candidate_only:
                        description: Enable/Disable backup selection from candidate-list only
                        type: bool
                      interface:
                        description: Specify Per-link LFA exclusion information
                        type: dict
                        suboptions:
                          bvi:
                            description: Specify Bridge-Group Virtual Interface
                            type: list
                            elements: int
                          bundle_ether:
                            description: Specify Aggregated Ethernet interface(s)
                            type: list
                            elements: int
                          pos_int:
                            description: Specify Aggregated pos interface(s)
                            type: list
                            elements: int
                          fast_ethernet:
                            description: Specify FastEthernet/IEEE 802.3 interface(s)
                            type: list
                            elements: str
                          fiftygige:
                            description: Specify FiftyGigE/IEEE 802.3 interface(s)
                            type: list
                            elements: str
                          fortygige:
                            description: Specify FortyGigE/IEEE 802.3 interface(s)
                            type: list
                            elements: str
                          fourhundredgige:
                            description: Specify FourHundredGigE/IEEE 802.3 interface(s)
                            type: list
                            elements: str
                          gigabitethernet:
                            description: Specify GigabitEthernet/IEEE 802.3 interface(s)
                            type: list
                            elements: str
                          hundredgige:
                            description: Specify HundredGigE/IEEE 802.3 interface(s)
                            type: list
                            elements: str
                          mgmteth:
                            description: Specify MgmtEth/IEEE 802.3 interface(s)
                            type: list
                            elements: str
                          multilink:
                            description: Specify Multilink network interface(s)
                            type: list
                            elements: str
                          pw_ether:
                            description: Specify PWHE Ethernet Interface
                            type: list
                            elements: int
                          pw_iw:
                            description: Specify PWHE VC11 IP Interworking Interface
                            type: list
                            elements: int
                          srp:
                            description: Specify SRP interface(s)
                            type: list
                            elements: str
                          serial:
                            description: Specify Serial network interface(s)
                            type: list
                            elements: str
                          tengige:
                            description: Specify TenGigabitEthernet/IEEE 802.3 interface(s)
                            type: list
                            elements: str
                          twentyfivegige:
                            description: Specify TwentyFiveGigabitEthernet/IEEE 802.3 interface(s)
                            type: list
                            elements: str
                          twohundredgige:
                            description: Specify TwoHundredGigE/IEEE 802.3 interface(s)
                            type: list
                            elements: str
                          nve:
                            description: Specify Network Virtualization Endpoint Interface(s)
                            type: list
                            elements: int
                          tunnel_ip:
                            description: Specify GRE/IPinIP Tunnel Interface(s)
                            type: list
                            elements: int
                          tunnel_ipsec:
                            description: Specify IPSec Tunnel interface(s)
                            type: list
                            elements: int
                          tunnel_mte:
                            description: Specify MPLS Traffic Engineering P2MP Tunnel interface(s)
                            type: list
                            elements: int
                          tunnel_mpls:
                            description: MPLS Transport Protocol Tunnel interface
                            type: int
              flood_reduction:
                description: Enable/Disable flood reduction
                type: bool
              hello_interval:
                description: Specify Time between HELLO packets
                type: int
              instance_id:
                description: Specify instance ID
                type: int
              mtu_ignore:
                description: Enable/Disable ignoring of MTU in DBD packets
                type: bool
              mpls_ldp_sync:
                description: Enable/Disable MPLS LDP Sync
                type: bool
              network:
                description: Specify Network type
                type: str
                choices: ["broadcast", "non-broadcast", "point-to-multipoint", "point-to-point"]
              nssa:
                description: NSSA settings for the area
                type: dict
                suboptions:
                  set:
                    description: Configure area as NSSA
                    type: bool
                  default_information_originate:
                    description: Originate default Type 7 LSA
                    type: dict
                    suboptions:
                      set:
                        description: Set nssa to default information originate
                        type: bool
                      metric:
                        description: ospfv3 default metric
                        type: int
                      metric_type:
                        description: Metric type for default routes
                        type: int
                  no_redistribution:
                    description: Do not send redistributed LSAs into NSSA area
                    type: bool
                  no_summary:
                    description: Do not send summary LSAs into NSSA area
                    type: bool
                  translate:
                    description: Translate LSA
                    type: dict
                    suboptions:
                      type7:
                        description: Translate from Type 7 to Type 5
                        type: dict
                        suboptions:
                          always:
                            description: Always translate LSAs
                            type: bool
                            required: true
              packet_size:
                description: Specify limit size of OSPFv3 packets
                type: int
              passive:
                description: Enable/Disable routing updates on an interface
                type: bool
              prefix_suppression:
                description: Hide all transit addresses on this interface
                type: bool
              priority:
                description: Specify Router priority
                type: int
              ranges:
                description: Summarize routes matching address/mask (border routers only)
                type: list
                elements: dict
                suboptions:
                  address:
                    description: IP in Prefix format (X:X::X/length)
                    type: str
                    required: True
                  cost:
                    description: Specify user specified metric for this range
                    type: int
                  advertise:
                    description: Advertise this range (default)
                    type: bool
                  not_advertise:
                    description: DoNotAdvertise this range
                    type: bool
              retransmit_interval:
                  description: Specify Delay between LSA retransmissions
                  type: int
              stub:
                description: Settings for configuring the area as a stub
                type: dict
                suboptions:
                  set:
                    description: Configure the area as a stub
                    type: bool
                  no_summary:
                    description: Do not send summary LSA into stub area
                    type: bool
              transmit_delay:
                description: Specify estimated time needed to send link-state update packet
                type: int
              virtual_link:
                description: Define a virtual link
                type: list
                elements: dict
                suboptions:
                  id:
                    description: Router-ID of virtual link neighbor (A.B.C.D)
                    type: str
                    required: True
                  authentication:
                    description: Enable authentication
                    type: dict
                    suboptions:
                      disable:
                        description: Do not authenticate OSPFv3 packets
                        type: bool
                        default: false
                      ipsec:
                        description: Specify IPSec AH authentication attributes
                        type: dict
                        suboptions:
                          spi:
                            description: Specify the Security Parameter Index value
                            type: int
                          algorithim_type:
                            description: Specify the type of algorithim
                            type: str
                            choices: ["md5", "sha1"]
                          key:
                            description: Specify key
                            type: str
                          clear_key:
                            description: Specify key in cleartext form
                            type: str
                          password_key:
                            description: Specify key in encrypted form
                            type: str
                  dead_interval:
                    description: Interval after which a neighbor is declared dead
                    type: int
                  hello_interval:
                    description: Time between HELLO packets
                    type: int
                  retransmit_interval:
                    description: Delay between LSA retransmissions
                    type: int
                  transmit_delay:
                    description: Link state transmit delay
                    type: int
                  encryption:
                    description: Encrypt and authenticate OSPFv3 packets
                    type: dict
                    suboptions:
                      disable:
                        description: Do not encrypt OSPFv3 packets
                        type: bool
                        default: false
                      ipsec:
                        description: Specify IPSec ESP encryption and authentication
                        type: dict
                        suboptions:
                          spi:
                            description: Specify the Security Parameter Index value
                            type: int
                          esp:
                            description: Specify encryption parameters
                            type: dict
                            suboptions:
                              triple_des:
                                description: This specify the triple DES algorithim
                                type: dict
                                suboptions:
                                  key:
                                    description: Cleartext 3DES key
                                    type: str
                                  clear_key:
                                    description: Specify 3DES key in cleartext form
                                    type: str
                                  password_key:
                                    description: Specify 3DES key in encrypted form
                                    type: str
                              aes:
                                description: This specify the aes algorithim
                                type: dict
                                suboptions:
                                  algorithim_type:
                                    description: Specify the bit encryption for aes algorithim
                                    type: str
                                    choices: ["192", "256"]
                                  key:
                                    description: Cleartext AES key
                                    type: str
                                  clear_key:
                                    description: Specify AES key in cleartext form
                                    type: str
                                  password_key:
                                    description: Specify AES key in encrypted form
                                    type: str
                              des:
                                description: This specify the des algorithim
                                type: dict
                                suboptions:
                                  key:
                                    description: Cleartext AES key
                                    type: str
                                  clear_key:
                                    description: Specify AES key in cleartext form
                                    type: str
                                  password_key:
                                    description: Specify AES key in encrypted form
                                    type: str
                              null_encryption:
                                description: Specify null encryption attributes
                                type: dict
                                suboptions:
                                  authentication:
                                    description: Specify authentication parameters
                                    type: dict
                                    suboptions:
                                      algorithim_type:
                                        description: Specify the type of algorithim
                                        type: str
                                        choices: ["md5", "sha1"]
                                      key:
                                        description: Specify key
                                        type: str
                                      clear_key:
                                        description: Specify key in cleartext form
                                        type: str
                                      password_key:
                                        description: Specify key in encrypted form
                                        type: str
              encryption:
                description: Encrypt and authenticate OSPFv3 packets
                type: dict
                suboptions:
                  disable:
                    description: Do not encrypt OSPFv3 packets
                    type: bool
                    default: false
                  ipsec:
                    description: Specify IPSec ESP encryption and authentication
                    type: dict
                    suboptions:
                      spi:
                        description: Specify the Security Parameter Index value
                        type: int
                      esp:
                        description: Specify encryption parameters
                        type: dict
                        suboptions:
                          triple_des:
                            description: This specify the triple DES algorithim
                            type: dict
                            suboptions:
                              key:
                                description: Cleartext 3DES key
                                type: str
                              clear_key:
                                description: Specify 3DES key in cleartext form
                                type: str
                              password_key:
                                description: Specify 3DES key in encrypted form
                                type: str
                          aes:
                            description: This specify the aes algorithim
                            type: dict
                            suboptions:
                              algorithim_type:
                                description: Specify the bit encryption for aes algorithim
                                type: str
                                choices: ["192", "256"]
                              key:
                                description: Cleartext AES key
                                type: str
                              clear_key:
                                description: Specify AES key in cleartext form
                                type: str
                              password_key:
                                description: Specify AES key in encrypted form
                                type: str
                          des:
                            description: This specify the des algorithim
                            type: dict
                            suboptions:
                              key:
                                description: Cleartext AES key
                                type: str
                              clear_key:
                                description: Specify AES key in cleartext form
                                type: str
                              password_key:
                                description: Specify AES key in encrypted form
                                type: str
                          null_encryption:
                            description: Specify null encryption attributes
                            type: dict
                            suboptions:
                              authentication:
                                description: Specify authentication parameters
                                type: dict
                                suboptions:
                                  algorithim_type:
                                    description: Specify the type of algorithim
                                    type: str
                                    choices: ["md5", "sha1"]
                                  key:
                                    description: Specify key
                                    type: str
                                  clear_key:
                                    description: Specify key in cleartext form
                                    type: str
                                  password_key:
                                    description: Specify key in encrypted form
                                    type: str
          capability:
            description: Enable specific OSPFv3 feature
            type: dict
            suboptions:
              type7:
                description: Specify type7 nssa capability
                type: dict
                suboptions:
                  prefer:
                    description: Prefer type7 externals over type5
                    type: bool
                  translate:
                    description: Translate type7 to type5
                    type: bool
          cost:
            description: Specify Interface cost
            type: int
          database_filter:
            description: Filter LSAs during synchronization and flooding
            type: dict
            suboptions:
              all_outgoing_lsa:
                description: Filter all outgoing LSA
                type: bool
          dead_interval:
            description: Interval after which a neighbor is declared dead
            type: int
          default_information_originate:
            description: Control distribution of default information
            type: dict
            suboptions:
              always:
                description: Always advertise default route
                type: bool
              metric:
                description: ospfv3 default metric
                type: int
              metric_type:
                description: ospfv3 metric type for default routes
                type: int
              route_policy:
                description: Apply route-policy to default-information origination
                type: str
              tag:
                description: Set tag for default route
                type: int
              set:
                description: Enable distribution of default route
                type: bool
          default_metric:
            description: Set metric of redistributed routes
            type: int
          demand_circuit:
            description: Enable/Disable ospfv3 demand circuit
            type: bool
          distance:
            description: Define an administrative distance
            type: dict
            suboptions:
              admin_distance:
                description: Administrative distance
                type: int
              ospfv3_distance:
                description: ospfv3 administrative distance
                type: dict
                suboptions:
                  external:
                    description: Distance for external routes
                    type: int
                  inter_area:
                    description: Distance for inter-area routes
                    type: int
                  intra_area:
                    description: Distance for intra-area routes
                    type: int
          distribute_list:
            description: Filter prefixes to/from RIB
            type: dict
            suboptions:
              prefix_list:
                description: Filter prefixes based on an IPv6 prefix-list
                type: list
                elements: str
                suboptions:
                  name:
                    description: Specify Prefix-list name
                    type: str
                  in:
                    description: Filter prefixes installed to RIB
                    type: bool
                  out:
                    description: Filter prefixes redistributed from RIB
                    type: bool
          encryption:
            description: Encrypt and authenticate OSPFv3 packets
            type: dict
            suboptions:
              disable:
                description: Do not encrypt OSPFv3 packets
                type: bool
                default: false
              ipsec:
                description: Specify IPSec ESP encryption and authentication
                type: dict
                suboptions:
                  spi:
                    description: Specify the Security Parameter Index value
                    type: int
                  esp:
                    description: Specify encryption parameters
                    type: dict
                    suboptions:
                      triple_des:
                        description: This specify the triple DES algorithim
                        type: dict
                        suboptions:
                          key:
                            description: Cleartext 3DES key
                            type: str
                          clear_key:
                            description: Specify 3DES key in cleartext form
                            type: str
                          password_key:
                            description: Specify 3DES key in encrypted form
                            type: str
                      aes:
                        description: This specify the aes algorithim
                        type: dict
                        suboptions:
                          algorithim_type:
                            description: Specify the bit encryption for aes algorithim
                            type: str
                            choices: ["192", "256"]
                          key:
                            description: Cleartext AES key
                            type: str
                          clear_key:
                            description: Specify AES key in cleartext form
                            type: str
                          password_key:
                            description: Specify AES key in encrypted form
                            type: str
                      des:
                        description: This specify the des algorithim
                        type: dict
                        suboptions:
                          key:
                            description: Cleartext AES key
                            type: str
                          clear_key:
                            description: Specify AES key in cleartext form
                            type: str
                          password_key:
                            description: Specify AES key in encrypted form
                            type: str
                      null_encryption:
                        description: Specify null encryption attributes
                        type: dict
                        suboptions:
                          authentication:
                            description: Specify authentication parameters
                            type: dict
                            suboptions:
                              algorithim_type:
                                description: Specify the type of algorithim
                                type: str
                                choices: ["md5", "sha1"]
                              key:
                                description: Specify key
                                type: str
                              clear_key:
                                description: Specify key in cleartext form
                                type: str
                              password_key:
                                description: Specify key in encrypted form
                                type: str
          fast_reroute:
            description: Specify IP Fast Reroute
            type: dict
            suboptions:
              disabled:
                description: Disable IP fast reroute
                type: bool
              per_link:
                description: Specify per-prefix computation
                type: dict
                suboptions:
                  information_type:
                    description: Specify per-link LFA exclusion or FRR LFA candidate information
                    type: str
                    choices: ["exclude", "lfa_candidate"]
                  use_candidate_only:
                    description: Enable/Disable backup selection from candidate-list only
                    type: bool
                  interface:
                    description: Specify Per-link LFA exclusion information
                    type: dict
                    suboptions:
                      bvi:
                        description: Specify Bridge-Group Virtual Interface
                        type: list
                        elements: int
                      bundle_ether:
                        description: Specify Aggregated Ethernet interface(s)
                        type: list
                        elements: int
                      pos_int:
                        description: Specify Aggregated pos interface(s)
                        type: list
                        elements: int
                      fast_ethernet:
                        description: Specify FastEthernet/IEEE 802.3 interface(s)
                        type: list
                        elements: str
                      fiftygige:
                        description: Specify FiftyGigE/IEEE 802.3 interface(s)
                        type: list
                        elements: str
                      fortygige:
                        description: Specify FortyGigE/IEEE 802.3 interface(s)
                        type: list
                        elements: str
                      fourhundredgige:
                        description: Specify FourHundredGigE/IEEE 802.3 interface(s)
                        type: list
                        elements: str
                      gigabitethernet:
                        description: Specify GigabitEthernet/IEEE 802.3 interface(s)
                        type: list
                        elements: str
                      hundredgige:
                        description: Specify HundredGigE/IEEE 802.3 interface(s)
                        type: list
                        elements: str
                      mgmteth:
                        description: Specify MgmtEth/IEEE 802.3 interface(s)
                        type: list
                        elements: str
                      multilink:
                        description: Specify Multilink network interface(s)
                        type: list
                        elements: str
                      pw_ether:
                        description: Specify PWHE Ethernet Interface
                        type: list
                        elements: int
                      pw_iw:
                        description: Specify PWHE VC11 IP Interworking Interface
                        type: list
                        elements: int
                      srp:
                        description: Specify SRP interface(s)
                        type: list
                        elements: str
                      serial:
                        description: Specify Serial network interface(s)
                        type: list
                        elements: str
                      tengige:
                        description: Specify TenGigabitEthernet/IEEE 802.3 interface(s)
                        type: list
                        elements: str
                      twentyfivegige:
                        description: Specify TwentyFiveGigabitEthernet/IEEE 802.3 interface(s)
                        type: list
                        elements: str
                      twohundredgige:
                        description: Specify TwoHundredGigE/IEEE 802.3 interface(s)
                        type: list
                        elements: str
                      nve:
                        description: Specify Network Virtualization Endpoint Interface(s)
                        type: list
                        elements: int
                      tunnel_ip:
                        description: Specify GRE/IPinIP Tunnel Interface(s)
                        type: list
                        elements: int
                      tunnel_ipsec:
                        description: Specify IPSec Tunnel interface(s)
                        type: list
                        elements: int
                      tunnel_mte:
                        description: Specify MPLS Traffic Engineering P2MP Tunnel interface(s)
                        type: list
                        elements: int
                      tunnel_mpls:
                        description: MPLS Transport Protocol Tunnel interface
                        type: int
              per_prefix:
                description: Specify per-prefix computation
                type: dict
                suboptions:
                  information_type:
                    description: Specify per_prefix LFA exclusion or FRR LFA candidate information
                    type: str
                    choices: ["exclude", "lfa_candidate"]
                  use_candidate_only:
                    description: Enable/Disable backup selection from candidate-list only
                    type: bool
                  interface:
                    description: Specify Per-link LFA exclusion information
                    type: dict
                    suboptions:
                      bvi:
                        description: Specify Bridge-Group Virtual Interface
                        type: list
                        elements: int
                      bundle_ether:
                        description: Specify Aggregated Ethernet interface(s)
                        type: list
                        elements: int
                      post_int:
                        description: Specify Aggregated pos interface(s)
                        type: list
                        elements: int
                      fast_ethernet:
                        description: Specify FastEthernet/IEEE 802.3 interface(s)
                        type: list
                        elements: str
                      fiftygige:
                        description: Specify FiftyGigE/IEEE 802.3 interface(s)
                        type: list
                        elements: str
                      fortygige:
                        description: Specify FortyGigE/IEEE 802.3 interface(s)
                        type: list
                        elements: str
                      fourhundredgige:
                        description: Specify FourHundredGigE/IEEE 802.3 interface(s)
                        type: list
                        elements: str
                      gigabitethernet:
                        description: Specify GigabitEthernet/IEEE 802.3 interface(s)
                        type: list
                        elements: str
                      hundredgige:
                        description: Specify HundredGigE/IEEE 802.3 interface(s)
                        type: list
                        elements: str
                      mgmteth:
                        description: Specify MgmtEth/IEEE 802.3 interface(s)
                        type: list
                        elements: str
                      multilink:
                        description: Specify Multilink network interface(s)
                        type: list
                        elements: str
                      pw_ether:
                        description: Specify PWHE Ethernet Interface
                        type: list
                        elements: int
                      pw_iw:
                        description: Specify PWHE VC11 IP Interworking Interface
                        type: list
                        elements: int
                      srp:
                        description: Specify SRP interface(s)
                        type: list
                        elements: str
                      serial:
                        description: Specify Serial network interface(s)
                        type: list
                        elements: str
                      tengige:
                        description: Specify TenGigabitEthernet/IEEE 802.3 interface(s)
                        type: list
                        elements: str
                      twentyfivegige:
                        description: Specify TwentyFiveGigabitEthernet/IEEE 802.3 interface(s)
                        type: list
                        elements: str
                      twohundredgige:
                        description: Specify TwoHundredGigE/IEEE 802.3 interface(s)
                        type: list
                        elements: str
                      nve:
                        description: Specify Network Virtualization Endpoint Interface(s)
                        type: list
                        elements: int
                      tunnel_ip:
                        description: Specify GRE/IPinIP Tunnel Interface(s)
                        type: list
                        elements: int
                      tunnel_ipsec:
                        description: Specify IPSec Tunnel interface(s)
                        type: list
                        elements: int
                      tunnel_mte:
                        description: Specify MPLS Traffic Engineering P2MP Tunnel interface(s)
                        type: list
                        elements: int
                      tunnel_mpls:
                        description: MPLS Transport Protocol Tunnel interface
                        type: int
          flood_reduction:
            description: Enable/Disable flood reduction
            type: bool
          graceful_restart:
            description: Enable Graceful-Restart
            type: dict
            suboptions:
              set:
                description: Set graceful restart
                type: bool
              helper_disable:
                description: Disable router's helper support level
                type: bool
              min_interval:
                description: Minimum interval between Graceful Restarts
                type: int
              max_interval:
                description: Maximum route lifetime following restart
                type: int
          hello_interval:
            description: Specify Time between HELLO packets
            type: int
          ignore_mospf_type6_lsa:
            description: Ignore MOSPF Type 6 LSA
            type: bool
          instance_id:
            description: Specify instance ID
            type: int
          log_adjacency_changes:
            description: Log adjacency state changes
            type: dict
            suboptions:
              set:
                description: Set log adjacency
                type: bool
              disable:
                description: Disable log adjacency changes
                type: bool
              detail:
                description: Log all state changes
                type: bool
          maximum:
            description: Set OSPFv3 limits
            type: dict
            suboptions:
              interfaces:
                description: Specify limit for number of interfaces
                type: int
              paths:
                description: Specify limit for number of paths
                type: int
              redistributed_prefixes:
                description: Specify limit for number of redistributed prefixes
                type: int
          mpls_ldp_sync:
            description: Enable/Disable MPLS LDP Sync
            type: bool
          mtu_ignore:
            description: Enable/Disable ignoring of MTU in DBD packets
            type: bool
          network:
            description: Specify Network type
            type: str
            choices: ["broadcast", "non-broadcast", "point-to-multipoint", "point-to-point"]
          nsr:
            description: Enable/Disable NSR for all VRFs in this process
            type: bool
          packet_size:
            description: Specify limit size of OSPFv3 packets
            type: int
          passive:
            description: Enable/Disable routing updates on an interface
            type: bool
          prefix_suppression:
            description: Hide all transit addresses on this interface
            type: bool
          priority:
            description: Specify Router priority
            type: int
          protocol_shutdown:
            description: Gracefully shutdown the OSPFv3 protocol
            type: bool
          redistribute:
            description: Redistribute information from another routing Protocol
            type: dict
            suboptions:
              application:
                description: Specify application routes
                type: list
                elements: dict
                suboptions:
                  id:
                    description: OnePK Application name
                    type: str
                    required: true
                  set:
                    description: Set application route
                    type: bool
                  metric:
                    description: Specify metric for redistributed routes
                    type: int
                  metric_type:
                    description: Specify OSPFv3 exterior metric type for redistributed routes
                    type: int
                  route_policy:
                    description: Apply route policy to redistribution
                    type: str
                  tag:
                    description: Set tag for routes redistributed into OSPFv3
                    type: int
              bgp:
                description: Specify bgp routes
                type: list
                elements: dict
                suboptions:
                  id:
                   description: BGP process name
                   type: int
                   required: true
                  set:
                    description: Set bgp route number
                    type: bool
                  metric:
                    description: Specify metric for redistributed routes
                    type: int
                  metric_type:
                    description: Specify OSPFv3 exterior metric type for redistributed routes
                    type: int
                  preserved_med:
                    description: Specify preserve med of BGP routes
                    type: str
                  route_policy:
                    description: Apply route policy to redistribution
                    type: str
                  tag:
                    description: Set tag for routes redistributed into OSPFv3
                    type: int
              connected:
                description: Specify connected routes
                type: dict
                suboptions:
                  set:
                    description: Set connected route
                    type: bool
                  metric:
                    description: Specify metric for redistributed routes
                    type: int
                  metric_type:
                    description: Specify OSPFv3 exterior metric type for redistributed routes
                    type: int
                  route_policy:
                    description: Apply route policy to redistribution
                    type: str
                  tag:
                    description: Set tag for routes redistributed into OSPFv3
                    type: int
              eigrp:
                description: Specify eigrp routes
                type: list
                elements: dict
                suboptions:
                  id:
                    description: EIGRP  process name
                    type: int
                    required: true
                  set:
                    description: Set bgp route number
                    type: bool
                  match:
                    description: Redistribution of EIGRP routes
                    type: str
                    choices: ["external", "internal"]
                  metric:
                    description: Specify metric for redistributed routes
                    type: int
                  metric_type:
                    description: Specify OSPFv3 exterior metric type for redistributed routes
                    type: int
                  route_policy:
                    description: Apply route policy to redistribution
                    type: str
                  tag:
                    description: Set tag for routes redistributed into OSPFv3
                    type: int
              isis:
                description: Specify IS-IS routes
                type: list
                elements: dict
                suboptions:
                  id:
                    description: IS-IS name
                    type: str
                    required: true
                  set:
                    description: Set IS-IS route number
                    type: bool
                  level:
                    description: Specify IS-IS level routes
                    type: str
                    choices: ["level-1", "level-1-2", "level-2"]
                  metric:
                    description: Specify metric for redistributed routes
                    type: int
                  metric_type:
                    description: Specify OSPFv3 exterior metric type for redistributed routes
                    type: int
                  route_policy:
                    description: Apply route policy to redistribution
                    type: str
                  tag:
                    description: Set tag for routes redistributed into OSPFv3
                    type: int
              mobile:
                description: Specify mobile routes
                type: dict
                suboptions:
                  set:
                    description: Set mobile route number
                    type: bool
                  metric:
                    description: Specify metric for redistributed routes
                    type: int
                  metric_type:
                    description: Specify OSPFv3 exterior metric type for redistributed routes
                    type: int
                  route_policy:
                    description: Apply route policy to redistribution
                    type: str
                  tag:
                    description: Set tag for routes redistributed into OSPFv3
                    type: int
              ospfv3:
                description: Specify ospfv3 routes
                type: list
                elements: dict
                suboptions:
                  id:
                    description: OSPFv3 process name
                    type: str
                    required: true
                  set:
                    description: Set ospfv3 route number
                    type: bool
                  match:
                    description: Redistribution of OSPFv3 routes
                    type: dict
                    suboptions:
                      external:
                        description: Redistribute OSPFv3 external routes
                        type: int
                        choices: ["1", "2"]
                      nssa_external:
                        description: Redistribute NSSA OSPFv3 external routes
                        type: int
                        choices: ["1", "2"]
                      internal:
                        description: Redistribute OSPFv3 internal routes
                        type: bool
                  metric:
                    description: Specify metric for redistributed routes
                    type: int
                  metric_type:
                    description: Specify OSPFv3 exterior metric type for redistributed routes
                    type: int
                  route_policy:
                    description: Apply route policy to redistribution
                    type: str
                  tag:
                    description: Set tag for routes redistributed into OSPFv3
                    type: int
              static:
                description: Specify static routes
                type: dict
                suboptions:
                  set:
                    description: Set static route
                    type: bool
                  metric:
                    description: Specify metric for redistributed routes
                    type: int
                  metric_type:
                    description: Specify OSPFv3 exterior metric type for redistributed routes
                    type: int
                  route_policy:
                    description: Apply route policy to redistribution
                    type: str
                  tag:
                    description: Set tag for routes redistributed into OSPFv3
                    type: int
              subscriber:
                description: Specify subscriber routes
                type: dict
                suboptions:
                  set:
                    description: Set static route
                    type: bool
                  metric:
                    description: Specify metric for redistributed routes
                    type: int
                  metric_type:
                    description: Specify OSPFv3 exterior metric type for redistributed routes
                    type: int
                  route_policy:
                    description: Apply route policy to redistribution
                    type: str
                  tag:
                    description: Set tag for routes redistributed into OSPFv3
                    type: int
          retransmit_interval:
            description: Delay between LSA retransmissions
            type: int
          router_id:
            description: ospfv3 router-id in IPv4 address format (A.B.C.D)
            type: str
          spf_prefix_priority:
            description: Specify SPF configuration
            type: dict
            suboptions:
              disable:
                description: Disable SPF prefix priority
                type: bool
              route_policy:
                description: Specify the route-policy to prioritize route install
                type: list
                elements: dict
                suboptions:
                  name:
                    description: Specify name of the policy
                    type: str
                  value:
                    description: Specify parameter values for the policy ()
                    type: str
          stub_router:
            description: Enter stub router configuration submode
            type: dict
            suboptions:
              router_lsa:
                description: Modify self originated router LSAs
                type: dict
                suboptions:
                  advertise_with:
                    description: Advertise LSAs with specified type
                    type: str
                    choices: ["max-metric", "r-bit", "v6-bit"]
                  always:
                    description: Force ospfv3 stub router mode unconditionally
                    type: bool
                  external_lsa:
                    description: Override External LSA metric in stub router mode
                    type: dict
                    suboptions:
                      set:
                        description: Set external lsa
                        type: bool
                      metric:
                        description: Metric to use while in stub router mode
                        type: int
                  include_stub:
                    description: Set maximum metric for stub links in stub router mode
                    type: bool
                  on_proc_migration:
                    description: Enter stub router mode on ospfv3 process migration
                    type: int
                  on_proc_restart:
                    description: Enter stub router mode on ospfv3 process restart
                    type: int
                  on_startup:
                    description: Enter stub router mode on startup
                    type: dict
                    suboptions:
                      time:
                        description: Time in seconds to stay in stub router mode
                        type: int
                      wait_for_bgp:
                        description: Exit stub router mode when BGP converges
                        type: bool
                  on_switchover:
                    description: Enter stub router mode on RP switchover
                    type: int
                  summary_lsa:
                    description: Override Summary LSA metric in stub router mode
                    type: dict
                    suboptions:
                      set:
                        description: Enable summary LSA
                        type: bool
                      metric:
                        description: Metric to use while in stub router mode
                        type: int
          summary_prefix:
            description: Configure IP address summaries
            type: list
            elements: dict
            suboptions:
              prefix:
                description: IP summary address/mask (A.B.C.D/prefix)
                type: str
                required: True
              not_advertise:
                description: Suppress routes that match the specified prefix/mask pair
                type: bool
              tag:
                description: Set tag
                type: int
          timers:
            description: Adjust routing timers
            type: dict
            suboptions:
              lsa_arrival:
                description: Specify LSA arrival timers
                type: int
              pacing:
                description: Specify pacing timers
                type: dict
                suboptions:
                  flood:
                    description: Flood pacing timer
                    type: int
                  lsa_group:
                    description: LSA group pacing timer
                    type: int
                  retransmission:
                    description: LSA group pacing timer
                    type: int
              throttle:
                description: Adjust throttle timers
                type: dict
                suboptions:
                  lsa:
                    description: Specify LSA throttle timers
                    type: dict
                    suboptions:
                      all_lsa_initial:
                        description: Delay to generate first occurrence of LSA in milliseconds
                        type: int
                      all_lsa_minimum:
                        description: Minimum delay between originating the same LSA in milliseconds
                        type: int
                  spf:
                    description: Specify SPF throttle timers
                    type: dict
                    suboptions:
                      spf_initial:
                        description: Delay to generate first occurrence of SPF in ms
                        type: int
                      spf_minimum:
                        description: Minimum delay between originating the same SPF in ms
                        type: int
          trace:
            description: Specify OSPF tracing options
            type: dict
            suboptions:
              size:
                description: Delete existing buffer and create one with N entries
                type: str
              value:
                description: Specify trace entry
                type: int
          transmit_delay:
            description: Estimated time needed to send link-state update packet
            type: int
  state:
    description:
      - The state the configuration should be left in
    type: str
    choices:
      - merged
      - replaced
      - overridden
      - deleted
      - gathered
      - rendered
      - parsed
    default: merged
"""
EXAMPLES = """
# Using merged

# Before state:
# -------------
#
# RP/0/RP0/CPU0:anton#show running-config router ospfv3
# Thu Jun 11 15:54:44.569 UTC
# % No such configuration item(s)
#

- name: Merge provided OSPFv3 configuration with the existing configuration
  cisco.iosxr.iosxr_ospfv3:
    config:
      processes:
        - process_id: 27
          areas:
            - area_id: 10
              hello_interval: 2
        - process_id: 26
          authentication:
            disable: true
        - process_id: 10
          areas:
            - area_id: 11
              default_cost: 5
              cost: 11
            - area_id: 22
              default_cost: 6
        - process_id: 30
          areas:
            - area_id: 11
              default_cost: 5
            - area_id: 22
              default_cost: 6
          cost: 2
          default_metric: 10
          transmit_delay: 2
          hello_interval: 1
          dead_interval: 2
          retransmit_interval: 2
          packet_size: 577
          priority: 1
          router_id: 2.2.2.2
          demand_circuit: true
          mtu_ignore: true
    state: merged

#
# Task Output:
# ------------
#
# before: {}
#
# - router ospfv3 10
#   - area 11 default-cost 5
#   - area 11 cost 11
#   - area 22 default-cost 6
#   - router ospfv3 26
#   - authentication disable
#   - router ospfv3 27
#   - area 10 hello-interval 2
#   - router ospfv3 30
#   - cost 2
#   - priority 1
#   - default-metric 10
#   - router-id 2.2.2.2
#   - demand-circuit
#   - packet-size 577
#   - transmit-delay 2
#   - dead-interval 2
#   - hello-interval 1
#   - retransmit-interval 2
#   - mtu-ignore
#   - area 11 default-cost 5
#   - area 22 default-cost 6
#
# after:
#     processes:
#     - areas:
#       - area_id: '11'
#         cost: 11
#         default_cost: 5
#       - area_id: '22'
#         default_cost: 6
#       process_id: '10'
#     - authentication:
#         disable: true
#       process_id: '26'
#     - areas:
#       - area_id: '10'
#         hello_interval: 2
#       process_id: '27'
#     - areas:
#       - area_id: '11'
#         default_cost: 5
#       - area_id: '22'
#         default_cost: 6
#       cost: 2
#       dead_interval: 2
#       default_metric: 10
#       demand_circuit: true
#       hello_interval: 1
#       mtu_ignore: true
#       packet_size: 577
#       priority: 1
#       process_id: '30'
#       retransmit_interval: 2
#       router_id: 2.2.2.2
#       transmit_delay: 2
#
# After state:
# ------------
#
# RP/0/RP0/CPU0:anton#show running-config router ospfv3
# router ospfv3 10
#  area 11
#   cost 11
#   default-cost 5
#  !
#  area 22
#   default-cost 6
#  !
# !
# router ospfv3 26
#  authentication disable
# !
# router ospfv3 27
#  area 10
#   hello-interval 2
#  !
# !
# router ospfv3 30
#  cost 2
#  priority 1
#  mtu-ignore
#  packet-size 577
#  dead-interval 2
#  retransmit-interval 2
#  demand-circuit
#  hello-interval 1
#  transmit-delay 2
#  router-id 2.2.2.2
#  default-metric 10
#  area 11
#   default-cost 5
#  !
#  area 22
#   default-cost 6
#  !
# !

# Using replaced
#
# Before state:
# -------------
#
#
# RP/0/RP0/CPU0:anton#show running-config router ospf
# router ospfv3 10
#  area 11
#   cost 11
#   default-cost 5
#  !
#  area 22
#   default-cost 6
#  !
# !
# router ospfv3 26
#  authentication disable
# !
# router ospfv3 27
#  area 10
#   hello-interval 2
#  !
# !
# router ospfv3 30
#  cost 2
#  priority 1
#  mtu-ignore
#  packet-size 577
#  dead-interval 2
#  retransmit-interval 2
#  demand-circuit
#  hello-interval 1
#  transmit-delay 2
#  router-id 2.2.2.2
#  default-metric 10
#  area 11
#   default-cost 5
#  !
#  area 22
#   default-cost 6
#  !
# !

- name: Replace OSPFv3 routes configurations from the device
  cisco.iosxr.iosxr_ospfv3:
    config:
      processes:
        - process_id: 27
          areas:
            - area_id: 10
              hello_interval: 2
            - area_id: 20
              cost: 2
              default_cost: 2
        - process_id: 26
          authentication:
            disable: true
    state: replaced

#
# Task Output:
# ------------
#
# before:
#     processes:
#     - areas:
#       - area_id: '11'
#         cost: 11
#         default_cost: 5
#       - area_id: '22'
#         default_cost: 6
#       process_id: '10'
#     - authentication:
#         disable: true
#       process_id: '26'
#     - areas:
#       - area_id: '10'
#         hello_interval: 2
#       process_id: '27'
#     - areas:
#       - area_id: '11'
#         default_cost: 5
#       - area_id: '22'
#         default_cost: 6
#       cost: 2
#       dead_interval: 2
#       default_metric: 10
#       demand_circuit: true
#       hello_interval: 1
#       mtu_ignore: true
#       packet_size: 577
#       priority: 1
#       process_id: '30'
#       retransmit_interval: 2
#       router_id: 2.2.2.2
#       transmit_delay: 2
#
# commands:
#   - router ospfv3 27
#   - area 20 default-cost 2
#   - area 20 cost 2
#
# after:
#     processes:
#     - areas:
#       - area_id: '11'
#         cost: 11
#         default_cost: 5
#       - area_id: '22'
#         default_cost: 6
#       process_id: '10'
#     - authentication:
#         disable: true
#       process_id: '26'
#     - areas:
#       - area_id: '10'
#         hello_interval: 2
#       - area_id: '20'
#         cost: 2
#         default_cost: 2
#       process_id: '27'
#     - areas:
#       - area_id: '11'
#         default_cost: 5
#       - area_id: '22'
#         default_cost: 6
#       cost: 2
#       dead_interval: 2
#       default_metric: 10
#       demand_circuit: true
#       hello_interval: 1
#       mtu_ignore: true
#       packet_size: 577
#       priority: 1
#       process_id: '30'
#       retransmit_interval: 2
#       router_id: 2.2.2.2
#       transmit_delay: 2
#
#
# After state:
# ------------
#
# RP/0/RP0/CPU0:anton(config)#do show running-config router ospfv3
# router ospfv3 10
#  area 11
#   cost 11
#   default-cost 5
#  !
#  area 22
#   default-cost 6
#  !
# !
# router ospfv3 26
#  authentication disable
# !
# router ospfv3 27
#  area 10
#   hello-interval 2
#  !
#  area 20
#   cost 2
#   default-cost 2
#  !
# !
# router ospfv3 30
#  cost 2
#  priority 1
#  mtu-ignore
#  packet-size 577
#  dead-interval 2
#  retransmit-interval 2
#  demand-circuit
#  hello-interval 1
#  transmit-delay 2
#  router-id 2.2.2.2
#  default-metric 10
#  area 11
#   default-cost 5
#  !
#  area 22
#   default-cost 6
#  !
# !

# Using overridden
#
# Before state:
# -------------
#
# router ospfv3 10
#  area 11
#   cost 11
#   default-cost 5
#  !
#  area 22
#   default-cost 6
#  !
# !
# router ospfv3 26
#  authentication disable
# !
# router ospfv3 27
#  area 10
#   hello-interval 2
#  !
#  area 20
#   cost 2
#   default-cost 2
#  !
# !
# router ospfv3 30
#  cost 2
#  priority 1
#  mtu-ignore
#  packet-size 577
#  dead-interval 2
#  retransmit-interval 2
#  demand-circuit
#  hello-interval 1
#  transmit-delay 2
#  router-id 2.2.2.2
#  default-metric 10
#  area 11
#   default-cost 5
#  !
#  area 22
#   default-cost 6
#  !
# !

- name: Override existing OSPFv3 configurations from the device
  cisco.iosxr.iosxr_ospfv3:
    config:
      processes:
        - process_id: 27
          areas:
            - area_id: 10
              hello_interval: 2
              authentication:
                disable: true
            - area_id: 20
              cost: 2
              default_cost: 2
              authentication:
                disable: true
        - process_id: 26
          areas:
            - area_id: 10
              hello_interval: 2
              authentication:
                disable: true
    state: overridden

#
# Task Output:
# ------------
#
# before:
#     processes:
#     - areas:
#       - area_id: '11'
#         cost: 11
#         default_cost: 5
#       - area_id: '22'
#         default_cost: 6
#       process_id: '10'
#     - authentication:
#         disable: true
#       process_id: '26'
#     - areas:
#       - area_id: '10'
#         hello_interval: 2
#       - area_id: '20'
#         cost: 2
#         default_cost: 2
#       process_id: '27'
#     - areas:
#       - area_id: '11'
#         default_cost: 5
#       - area_id: '22'
#         default_cost: 6
#       cost: 2
#       dead_interval: 2
#       default_metric: 10
#       demand_circuit: true
#       hello_interval: 1
#       mtu_ignore: true
#       packet_size: 577
#       priority: 1
#       process_id: '30'
#       retransmit_interval: 2
#       router_id: 2.2.2.2
#       transmit_delay: 2
#
# commands:
#   - router ospfv3 10
#   - no area 11 default-cost 5
#   - no area 11 cost 11
#   - no area 22 default-cost 6
#   - router ospfv3 30
#   - no cost 2
#   - no priority 1
#   - no default-metric 10
#   - no router-id 2.2.2.2
#   - no demand-circuit
#   - no packet-size 577
#   - no transmit-delay 2
#   - no dead-interval 2
#   - no hello-interval 1
#   - no retransmit-interval 2
#   - no mtu-ignore
#   - no area 11 default-cost 5
#   - no area 22 default-cost 6
#   - router ospfv3 27
#   - area 10 authentication disable
#   - area 20 authentication disable
#   - router ospfv3 26
#   - no authentication disable
#   - area 10 authentication disable
#   - area 10 hello-interval 2
#
# after:
#     processes:
#     - process_id: '10'
#     - areas:
#       - area_id: '10'
#         authentication:
#           disable: true
#         hello_interval: 2
#       process_id: '26'
#     - areas:
#       - area_id: '10'
#         authentication:
#           disable: true
#         hello_interval: 2
#       - area_id: '20'
#         authentication:
#           disable: true
#         cost: 2
#         default_cost: 2
#       process_id: '27'
#     - process_id: '30'
#
# After state:
# ------------
#
# RP/0/RP0/CPU0:anton#show running-config router ospfv3
# router ospfv3 10
#  area 11
#  !
#  area 22
#  !
# !
# router ospfv3 26
#  area 10
#   authentication disable
#   hello-interval 2
#  !
# !
# router ospfv3 27
#  area 10
#   authentication disable
#   hello-interval 2
#  !
#  area 20
#   cost 2
#   authentication disable
#   default-cost 2
#  !
# !
# router ospfv3 30
#  area 11
#  !
#  area 22
#  !
# !

# Using deleted
#
# Before state:
# -------------
#
#
# RP/0/RP0/CPU0:anton#show running-config router ospfv3
# router ospfv3 10
#  area 11
#   cost 11
#   default-cost 5
#  !
#  area 22
#   default-cost 6
#  !
# !
# router ospfv3 26
#  authentication disable
#  area 10
#   authentication disable
#   hello-interval 2
#  !
# !
# router ospfv3 27
#  area 10
#   authentication disable
#   hello-interval 2
#  !
#  area 20
#   cost 2
#   authentication disable
#   default-cost 2
#  !
# !
# router ospfv3 30
#  cost 2
#  priority 1
#  mtu-ignore
#  packet-size 577
#  dead-interval 2
#  retransmit-interval 2
#  demand-circuit
#  hello-interval 1
#  transmit-delay 2
#  router-id 2.2.2.2
#  default-metric 10
#  area 11
#   default-cost 5
#  !
#  area 22
#   default-cost 6
#  !
# !

- name: Deleted existing OSPFv3 configurations from the device
  cisco.iosxr.iosxr_ospfv3:
    config:
      processes:
        - process_id: '10'
        - process_id: '26'
        - process_id: '27'
        - process_id: '30'
    state: deleted


#
# Task Output:
# ------------
#
# before:
#     processes:
#     - areas:
#       - area_id: '11'
#         cost: 11
#         default_cost: 5
#       - area_id: '22'
#         default_cost: 6
#       process_id: '10'
#     - areas:
#       - area_id: '10'
#         authentication:
#           disable: true
#         hello_interval: 2
#       authentication:
#         disable: true
#       process_id: '26'
#     - areas:
#       - area_id: '10'
#         authentication:
#           disable: true
#         hello_interval: 2
#       - area_id: '20'
#         authentication:
#           disable: true
#         cost: 2
#         default_cost: 2
#       process_id: '27'
#     - areas:
#       - area_id: '11'
#         default_cost: 5
#       - area_id: '22'
#         default_cost: 6
#       cost: 2
#       dead_interval: 2
#       default_metric: 10
#       demand_circuit: true
#       hello_interval: 1
#       mtu_ignore: true
#       packet_size: 577
#       priority: 1
#       process_id: '30'
#       retransmit_interval: 2
#       router_id: 2.2.2.2
#       transmit_delay: 2
#
# commands:
#   - router ospfv3 10
#   - no area 11 default-cost 5
#   - no area 11 cost 11
#   - no area 22 default-cost 6
#   - router ospfv3 26
#   - no authentication disable
#   - no area 10 authentication disable
#   - no area 10 hello-interval 2
#   - router ospfv3 27
#   - no area 10 authentication disable
#   - no area 10 hello-interval 2
#   - no area 20 authentication disable
#   - no area 20 default-cost 2
#   - no area 20 cost 2
#   - router ospfv3 30
#   - no cost 2
#   - no priority 1
#   - no default-metric 10
#   - no router-id 2.2.2.2
#   - no demand-circuit
#   - no packet-size 577
#   - no transmit-delay 2
#   - no dead-interval 2
#   - no hello-interval 1
#   - no retransmit-interval 2
#   - no mtu-ignore
#   - no area 11 default-cost 5
#   - no area 22 default-cost 6
#
# after:
#     processes:
#     - process_id: '10'
#     - process_id: '26'
#     - process_id: '27'
#     - process_id: '30'
#
# After state:
# ------------
#
# RP/0/RP0/CPU0:anton(config)#show running-config router ospfv3
# router ospfv3 10
#  area 11
#  !
#  area 22
#  !
# !
# router ospfv3 26
#  area 10
#  !
# !
# router ospfv3 27
#  area 10
#  !
#  area 20
#  !
# !
# router ospfv3 30
#  area 11
#  !
#  area 22
#  !
# !

# Using deleted
#
# Before state:
# -------------
#
#
# RP/0/RP0/CPU0:anton#show running-config router ospfv3
# router ospfv3 10
#  area 11
#   cost 11
#   default-cost 5
#  !
#  area 22
#   default-cost 6
#  !
# !
# router ospfv3 26
#  authentication disable
#  area 10
#   authentication disable
#   hello-interval 2
#  !
# !
# router ospfv3 27
#  area 10
#   authentication disable
#   hello-interval 2
#  !
#  area 20
#   cost 2
#   authentication disable
#   default-cost 2
#  !
# !
# router ospfv3 30
#  cost 2
#  priority 1
#  mtu-ignore
#  packet-size 577
#  dead-interval 2
#  retransmit-interval 2
#  demand-circuit
#  hello-interval 1
#  transmit-delay 2
#  router-id 2.2.2.2
#  default-metric 10
#  area 11
#   default-cost 5
#  !
#  area 22
#   default-cost 6
#  !
# !

- name: Deleted existing OSPFv3 configurations from the device
  cisco.iosxr.iosxr_ospfv3:
    config:
    state: deleted

#
# Task Output:
# ------------
#
# before:
#     processes:
#     - areas:
#       - area_id: '11'
#         cost: 11
#         default_cost: 5
#       - area_id: '22'
#         default_cost: 6
#       process_id: '10'
#     - areas:
#       - area_id: '10'
#         authentication:
#           disable: true
#         hello_interval: 2
#       authentication:
#         disable: true
#       process_id: '26'
#     - areas:
#       - area_id: '10'
#         authentication:
#           disable: true
#         hello_interval: 2
#       - area_id: '20'
#         authentication:
#           disable: true
#         cost: 2
#         default_cost: 2
#       process_id: '27'
#     - areas:
#       - area_id: '11'
#         default_cost: 5
#       - area_id: '22'
#         default_cost: 6
#       cost: 2
#       dead_interval: 2
#       default_metric: 10
#       demand_circuit: true
#       hello_interval: 1
#       mtu_ignore: true
#       packet_size: 577
#       priority: 1
#       process_id: '30'
#       retransmit_interval: 2
#       router_id: 2.2.2.2
#       transmit_delay: 2
#
# commands:
#   - router ospfv3 10
#   - no area 11 default-cost 5
#   - no area 11 cost 11
#   - no area 22 default-cost 6
#   - router ospfv3 26
#   - no authentication disable
#   - no area 10 authentication disable
#   - no area 10 hello-interval 2
#   - router ospfv3 27
#   - no area 10 authentication disable
#   - no area 10 hello-interval 2
#   - no area 20 authentication disable
#   - no area 20 default-cost 2
#   - no area 20 cost 2
#   - router ospfv3 30
#   - no cost 2
#   - no priority 1
#   - no default-metric 10
#   - no router-id 2.2.2.2
#   - no demand-circuit
#   - no packet-size 577
#   - no transmit-delay 2
#   - no dead-interval 2
#   - no hello-interval 1
#   - no retransmit-interval 2
#   - no mtu-ignore
#   - no area 11 default-cost 5
#   - no area 22 default-cost 6
#
# after:
#     processes:
#     - process_id: '10'
#     - process_id: '26'
#     - process_id: '27'
#     - process_id: '30'
#
# After state:
# ------------
#
# RP/0/RP0/CPU0:anton(config)#show running-config router ospfv3
# router ospfv3 10
#  area 11
#  !
#  area 22
#  !
# !
# router ospfv3 26
#  area 10
#  !
# !
# router ospfv3 27
#  area 10
#  !
#  area 20
#  !
# !
# router ospfv3 30
#  area 11
#  !
#  area 22
#  !
# !


# Using parsed
#
# parsed.cfg
# ------------
# router ospfv3 10
#  area 11
#   cost 11
#   default-cost 5
#  !
#  area 22
#   default-cost 6
#  !
# !
# router ospfv3 26
#  authentication disable
# !
# router ospfv3 27
#  area 10
#   hello-interval 2
#  !
# !
# router ospfv3 30
#  router-id 2.2.2.2
#  cost 2
#  packet-size 577
#  priority 1
#  mtu-ignore
#  dead-interval 2
#  retransmit-interval 2
#  demand-circuit
#  hello-interval 1
#  transmit-delay 2
#  default-metric 10
#  area 11
#   default-cost 5
#  !
#  area 22
#   default-cost 6
#  !
# !
- name: Parsed the device configuration to get output commands
  cisco.iosxr.iosxr_ospfv3:
    running_config: "{{ lookup('file', './parsed.cfg') }}"
    state: parsed
#
#
# Task Output:
# ------------
#
# parsed:
#     processes:
#     - areas:
#       - area_id: '11'
#         cost: 11
#         default_cost: 5
#       - area_id: '22'
#         default_cost: 6
#       process_id: '10'
#     - authentication:
#         disable: true
#       process_id: '26'
#     - areas:
#       - area_id: '10'
#         hello_interval: 2
#       process_id: '27'
#     - areas:
#       - area_id: '11'
#         default_cost: 5
#       - area_id: '22'
#         default_cost: 6
#       cost: 2
#       dead_interval: 2
#       default_metric: 10
#       demand_circuit: true
#       hello_interval: 1
#       mtu_ignore: true
#       packet_size: 577
#       priority: 1
#       process_id: '30'
#       retransmit_interval: 2
#       router_id: 2.2.2.2
#       transmit_delay: 2
#
# Using rendered
#
- name: Render the commands for provided  configuration
  cisco.iosxr.iosxr_ospfv3:
    config:
      processes:
        - process_id: 27
          areas:
            - area_id: 10
              hello_interval: 2
        - process_id: 26
          authentication:
            disable: true
        - process_id: 10
          areas:
            - area_id: 11
              default_cost: 5
              cost: 11
            - area_id: 22
              default_cost: 6
        - process_id: 30
          areas:
            - area_id: 11
              default_cost: 5
            - area_id: 22
              default_cost: 6
          cost: 2
          default_metric: 10
          transmit_delay: 2
          hello_interval: 1
          dead_interval: 2
          retransmit_interval: 2
          packet_size: 577
          priority: 1
          router_id: '2.2.2.2'
          demand_circuit: true
          mtu_ignore: true
    state: rendered

#
#
# Task Output:
# ------------
#
# rendered:
#   - router ospfv3 27
#   - area 10 hello-interval 2
#   - router ospfv3 26
#   - authentication disable
#   - router ospfv3 10
#   - area 11 default-cost 5
#   - area 11 cost 11
#   - area 22 default-cost 6
#   - router ospfv3 30
#   - cost 2
#   - priority 1
#   - default-metric 10
#   - router-id 2.2.2.2
#   - demand-circuit
#   - packet-size 577
#   - transmit-delay 2
#   - dead-interval 2
#   - hello-interval 1
#   - retransmit-interval 2
#   - mtu-ignore
#   - area 11 default-cost 5
#   - area 22 default-cost 6


# Using gathered
#
# Before state:
# -------------
#
# RP/0/RP0/CPU0:anton#show running-config router ospfv3
# router ospfv3 10
#  area 11
#   cost 11
#   default-cost 5
#  !
#  area 22
#   default-cost 6
#  !
# !
# router ospfv3 26
#  authentication disable
#  area 10
#  !
# !
# router ospfv3 27
#  area 10
#   hello-interval 2
#  !
#  area 20
#  !
# !
# router ospfv3 30
#  cost 2
#  priority 1
#  mtu-ignore
#  packet-size 577
#  dead-interval 2
#  retransmit-interval 2
#  demand-circuit
#  hello-interval 1
#  transmit-delay 2
#  router-id 2.2.2.2
#  default-metric 10
#  area 11
#   default-cost 5
#  !
#  area 22
#   default-cost 6
#  !
# !

- name: Gather ospfv3 routes configuration
  cisco.iosxr.iosxr_ospfv3:
    state: gathered
#
#
# Task Output:
# ------------
#
# gathered:
#     processes:
#     - areas:
#       - area_id: '11'
#         cost: 11
#         default_cost: 5
#       - area_id: '22'
#         default_cost: 6
#       process_id: '10'
#     - authentication:
#         disable: true
#       process_id: '26'
#     - areas:
#       - area_id: '10'
#         hello_interval: 2
#       process_id: '27'
#     - areas:
#       - area_id: '11'
#         default_cost: 5
#       - area_id: '22'
#         default_cost: 6
#       cost: 2
#       dead_interval: 2
#       default_metric: 10
#       demand_circuit: true
#       hello_interval: 1
#       mtu_ignore: true
#       packet_size: 577
#       priority: 1
#       process_id: '30'
#       retransmit_interval: 2
#       router_id: 2.2.2.2
#       transmit_delay: 2
#
"""
RETURN = """
before:
  description: The configuration prior to the model invocation.
  returned: always
  type: dict
  sample: >
    The configuration returned will always be in the same format
     of the parameters above.
after:
  description: The resulting configuration model invocation.
  returned: when changed
  type: dict
  sample: >
    The configuration returned will always be in the same format
     of the parameters above.
commands:
  description: The set of commands pushed to the remote device.
  returned: always
  type: list
  sample:
  - router ospfv3 10
  - area 11 default-cost 5
  - area 11 cost 11
  - area 22 default-cost 6

rendered:
  description: The provided configuration in the task rendered in device-native format (offline).
  returned: when I(state) is C(rendered)
  type: list
  sample:
  - router ospfv3 27
  - area 10 hello-interval 2
gathered:
  description: Facts about the network resource gathered from the remote device as structured data.
  returned: when I(state) is C(gathered)
  type: dict
  sample: >
    This output will always be in the same format as the
    module argspec.
parsed:
  description: The device native config provided in I(running_config) option parsed into structured data as per module argspec.
  returned: when I(state) is C(parsed)
  type: dict
  sample: >
    This output will always be in the same format as the
    module argspec.
"""

from ansible.module_utils.basic import AnsibleModule

from ansible_collections.cisco.iosxr.plugins.module_utils.network.iosxr.argspec.ospfv3.ospfv3 import (
    Ospfv3Args,
)
from ansible_collections.cisco.iosxr.plugins.module_utils.network.iosxr.config.ospfv3.ospfv3 import (
    Ospfv3,
)


def main():
    """
    Main entry point for module execution

    :returns: the result form module invocation
    """
    module = AnsibleModule(
        argument_spec=Ospfv3Args.argument_spec,
        mutually_exclusive=[["config", "running_config"]],
        required_if=[
            ["state", "merged", ["config"]],
            ["state", "replaced", ["config"]],
            ["state", "overridden", ["config"]],
            ["state", "rendered", ["config"]],
            ["state", "parsed", ["running_config"]],
        ],
        supports_check_mode=True,
    )

    result = Ospfv3(module).execute_module()
    module.exit_json(**result)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: ec2_vpc_vgw_info
version_added: 1.0.0
version_added_collection: community.aws
short_description: Gather information about virtual gateways in AWS
description:
  - Gather information about virtual gateways (VGWs) in AWS.
options:
  filters:
    description:
      - A dict of filters to apply. Each dict item consists of a filter key and a filter value.
        See U(https://docs.aws.amazon.com/AWSEC2/latest/APIReference/API_DescribeVpnGateways.html) for possible filters.
    type: dict
    default: {}
  vpn_gateway_ids:
    description: One or more virtual private gateway IDs.
    type: list
    elements: str
author:
  - "Nick Aslanidis (@naslanidis)"
extends_documentation_fragment:
  - amazon.aws.common.modules
  - amazon.aws.region.modules
  - amazon.aws.boto3
"""

EXAMPLES = r"""
# Note: These examples do not set authentication details, see the AWS Guide for details.

- name: Gather information about all virtual gateways for an account or profile
  amazon.aws.ec2_vpc_vgw_info:
    region: ap-southeast-2

- name: Gather information about a filtered list of Virtual Gateways
  amazon.aws.ec2_vpc_vgw_info:
    region: ap-southeast-2
    profile: production
    filters:
      "tag:Name": "main-virt-gateway"

- name: Gather information about a specific virtual gateway by VpnGatewayIds
  amazon.aws.ec2_vpc_vgw_info:
    region: ap-southeast-2
    profile: production
    vpn_gateway_ids: vgw-c432f6a7
"""

RETURN = r"""
virtual_gateways:
    description: Information about one or more virtual private gateways.
    returned: always
    type: list
    elements: dict
    contains:
      vpn_gateway_id:
        description: The ID of the virtual private gateway.
        type: str
        returned: success
        example: "vgw-0123456789abcdef0"
      state:
        description: Informtion about the current state of the virtual private gateway.
        type: str
        returned: success
        example: "available"
      type:
        description: Information about type of VPN connection the virtual private gateway supports.
        type: str
        returned: success
        example: "ipsec.1"
      vpc_attachments:
        description: Information about the VPCs attached to the virtual private gateway.
        type: list
        elements: dict
        returned: success
        contains:
          state:
            description: The current state of the attachment.
            type: str
            returned: success
            example: "available"
          vpc_id:
            description: The ID of the VPC.
            type: str
            returned: success
            example: "vpc-12345678901234567"
      tags:
        description:
          - A list of dictionaries representing the tags attached to the virtual private gateway.
          - Represents the same details as RV(virtual_gateways.resource_tags).
        type: list
        elements: dict
        returned: success
        contains:
          key:
            description: The key of the tag.
            type: str
            returned: success
            example: "MyKey"
          value:
            description: The value of the tag.
            type: str
            returned: success
            example: "MyValue"
      resource_tags:
        description:
          - A dictionary representing the tags attached to the VGW.
          - Represents the same details as RV(virtual_gateways.tags).
        type: dict
        returned: success
        example: {
                    "MyKey": "MyValue",
                    "Env": "Dev_Test_01"
                  }
"""

from ansible.module_utils.common.dict_transformations import camel_dict_to_snake_dict

from ansible_collections.amazon.aws.plugins.module_utils.ec2 import AnsibleEC2Error
from ansible_collections.amazon.aws.plugins.module_utils.ec2 import describe_vpn_gateways
from ansible_collections.amazon.aws.plugins.module_utils.modules import AnsibleAWSModule
from ansible_collections.amazon.aws.plugins.module_utils.tagging import boto3_tag_list_to_ansible_dict
from ansible_collections.amazon.aws.plugins.module_utils.transformation import ansible_dict_to_boto3_filter_list


def get_virtual_gateway_info(virtual_gateway):
    tags = virtual_gateway.get("Tags", [])
    resource_tags = boto3_tag_list_to_ansible_dict(tags)
    virtual_gateway_info = dict(
        VpnGatewayId=virtual_gateway["VpnGatewayId"],
        State=virtual_gateway["State"],
        Type=virtual_gateway["Type"],
        VpcAttachments=virtual_gateway["VpcAttachments"],
        Tags=tags,
        ResourceTags=resource_tags,
    )
    return virtual_gateway_info


def list_virtual_gateways(client, module):
    params = dict()
    vpn_gateway_ids = module.params.get("vpn_gateway_ids")
    filters = module.params.get("filters")

    if filters:
        params["Filters"] = ansible_dict_to_boto3_filter_list(filters)
    if vpn_gateway_ids:
        params["VpnGatewayIds"] = vpn_gateway_ids

    try:
        all_virtual_gateways = describe_vpn_gateways(client, **params)
    except AnsibleEC2Error as e:
        module.fail_json_aws_error(e)

    return [
        camel_dict_to_snake_dict(get_virtual_gateway_info(vgw), ignore_list=["ResourceTags"])
        for vgw in all_virtual_gateways
    ]


def main():
    argument_spec = dict(
        filters=dict(type="dict", default=dict()),
        vpn_gateway_ids=dict(type="list", default=None, elements="str"),
    )

    module = AnsibleAWSModule(argument_spec=argument_spec, supports_check_mode=True)

    connection = module.client("ec2")
    results = list_virtual_gateways(connection, module)

    module.exit_json(virtual_gateways=results)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: network_devices_unassign_from_site_apply_v1
short_description: Resource module for Network Devices Unassign From Site Apply V1
description:
- Manage operation create of the resource Network Devices Unassign From Site Apply V1.
- >
   Unassign unprovisioned network devices from their site. If device controllability is enabled, it will be triggered
   once device unassigned from site successfully. Device Controllability can be enabled/disabled using
   `/dna/intent/api/v1/networkDevices/deviceControllability/settings`.
version_added: '6.15.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  deviceIds:
    description: Network device Ids.
    elements: str
    type: list
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Site Design UnassignNetworkDevicesFromSitesV1
  description: Complete reference of the UnassignNetworkDevicesFromSitesV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!unassign-network-devices-from-sites
notes:
  - SDK Method used are
    site_design.SiteDesign.unassign_network_devices_from_sites_v1,

  - Paths used are
    post /dna/intent/api/v1/networkDevices/unassignFromSite/apply,

"""

EXAMPLES = r"""
- name: Create
  cisco.dnac.network_devices_unassign_from_site_apply_v1:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    deviceIds:
    - string

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "version": "string",
      "response": {
        "url": "string",
        "taskId": "string"
      }
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)


DOCUMENTATION = r"""
---
module: images_addon_images_info
short_description: Information module for Images Addon Images Info
description:
- This module represents an alias of the module images_addon_images_v1_info
version_added: '6.15.0'
extends_documentation_fragment:
  - cisco.dnac.module_info
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
  id:
    description:
    - >
      Id path parameter. Software image identifier. Check `/dna/intent/api/v1/images?hasAddonImages=true` API to
      get the same.
    type: str
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Software Image Management (SWIM) RetrieveApplicableAddOnImagesForTheGivenSoftwareImageV1
  description: Complete reference of the RetrieveApplicableAddOnImagesForTheGivenSoftwareImageV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!retrieve-applicable-add-on-images-for-the-given-software-image
notes:
  - SDK Method used are
    software_image_management_swim.SoftwareImageManagementSwim.retrieve_applicable_add_on_images_for_the_given_software_image_v1,

  - Paths used are
    get /dna/intent/api/v1/images/{id}/addonImages,
  - It should be noted that this module is an alias of images_addon_images_v1_info

"""

EXAMPLES = r"""
- name: Get all Images Addon Images Info
  cisco.dnac.images_addon_images_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    id: string
  register: result

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": [
        {
          "id": "string",
          "imported": true,
          "name": "string",
          "version": "string",
          "imageType": "string",
          "recommended": "string",
          "ciscoLatest": true,
          "integrityStatus": "string",
          "isAddonImage": true,
          "hasAddonImages": true,
          "goldenTaggingDetails": [
            {
              "deviceRoles": [
                "string"
              ],
              "deviceTags": [
                "string"
              ],
              "inheritedSiteId": "string",
              "inheritedSiteName": "string"
            }
          ],
          "productNames": [
            {
              "id": "string",
              "productName": "string",
              "productNameOrdinal": 0,
              "supervisorProductName": "string",
              "supervisorProductNameOrdinal": 0
            }
          ],
          "isGoldenTagged": true
        }
      ],
      "version": "string"
    }
"""



#!/usr/bin/python
#
# Copyright (c) 2024 xuzhang3 (@xuzhang3), Fred-sun (@Fred-sun)
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: azure_rm_sqlmidatabase
version_added: "2.4.0"
short_description: Manage SQL Managed Instance databases
description:
    - Manage SQL Managed Instance databases.

options:
    resource_group:
        description:
            - The name of the resource group that contains the resource.
        type: str
        required: true
    managed_instance_name:
        description:
            - The name of the SQL managed instance.
        type: str
        required: true
    database_name:
        description:
            - The name of the SQL managed instance database.
        type: str
        required: true
    collation:
        description:
            - The collation of the Azure SQL Managed Database collation to use.
            - For example C(SQL_Latin1_General_CP1_CI_AS) or C(Latin1_General_100_CS_AS_SC).
        type: str
    location:
        description:
            - The resource location.
        type: str
    state:
        description:
            - State of the SQL Managed Database.
            - Use C(present) to create or update a automation runbook and use C(absent) to delete.
        type: str
        default: present
        choices:
            - present
            - absent
extends_documentation_fragment:
    - azure.azcollection.azure
    - azure.azcollection.azure_tags

author:
    - xuzhang3 (@xuzhang3)
    - Fred-sun (@Fred-sun)
'''

EXAMPLES = '''
- name: Create a SQL managed instance database
  azure_rm_sqlmidatabase:
    resource_group: testrg
    managed_instance_name: testinstancename
    database_name: newdatabase
    collation: SQL_Latin1_General_CP1_CI_AS
    location: eastus
    tags:
      key2: value2

- name: Delete the SQL managed instance database
  azure_rm_sqlmidatabase:
    resource_group: testrg
    managed_instance_name: testinstancename
    database_name: newdatabase
    state: absent
'''

RETURN = '''
database:
    description:
        - A dictionary containing facts for SQL Managed Instance database info.
    returned: always
    type: complex
    contains:
        auto_complete_restore:
            description:
                - Whether to auto complete restore of this managed database.
            type: bool
            returned: always
            sample: null
        catalog_collation:
            description:
                - Collation of the metadata catalog.
            type: str
            returned: always
            sample: null
        create_mode:
            description:
                - Managed database create mode.
            type: str
            returned: always
            sample: null
        create_date:
            description:
                - Creation date of the database.
            type: str
            returned: always
            sample: "2024-05-06T23:59:49.770Z"
        database_name:
            description:
                - The sql mi databse name.
            type: str
            returned: always
            sample: fredtest
        default_secondary_location:
            description:
                - Geo paired region.
            type: str
            returned: always
            sample: westus
        id:
            description:
                - The resource ID.
            type: str
            returned: always
            sample: "/subscriptions/xxx-xxxx/resourceGroups/testRG/providers/Microsoft.Sql/managedInstances/fredsqlmin/databases/fredtest"
        last_backup_name:
            description:
                - Last backup file name for restore of this managed database.
            type: str
            returned: always
            sample: null
        location:
            description:
                - The resource's location.
            type: str
            returned: always
            sample: eastus
        long_term_retention_backup_resource_id:
            description:
                - The name of the Long Term Retention backup to be used for restore of this managed database.
            type: str
            returned: always
            sample: null
        managed_instance_name:
            description:
                - The name of the SQL managed instance.
            type: str
            returned: always
            sample: fredsqlmin
        recoverable_database_id:
            description:
                - The resource identifier of the recoverable database associated with the database.
            type: str
            returned: always
            sample: null
        resource_group:
            description:
                - The resource's resource group.
            type: str
            returned: always
            sample: testRG
        restorable_dropped_database_id:
            description:
                - The restorable dropped database resource id.
            type: str
            returned: always
            sample: null
        restore_point_in_time:
            description:
                - Specifies the point in time (ISO8601 format) of the source database.
            type: str
            returned: always
            sample: null
        source_database_id:
            description:
                - The resource identifier of the source database associated with create operation of this database.
            type: str
            returned: always
            sample: null
        status:
            description:
                - Status of the database.
            type: str
            returned: always
            sample: online
        storage_container_sas_token:
            description:
                - Specifies the storage container sas token.
            type: str
            returned: always
            sample: null
        storage_container_uri:
            description:
                - Specifies the uri of the storage container where backups for this restore are stopped.
            type: str
            returned: always
            sample: null
        tags:
            description:
                - The resource's tags
            type: str
            returned: always
            sample: {key1: value1}
        type:
            description:
                - The resource type.
            type: str
            returned: always
            sample: "Microsoft.Sql/managedInstances/databases"

'''

from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase

try:
    from azure.core.exceptions import HttpResponseError
except ImportError:
    # This is handled in azure_rm_common
    pass


class AzureRMSqlMIDatabase(AzureRMModuleBase):
    def __init__(self):
        # define user inputs into argument
        self.module_arg_spec = dict(
            resource_group=dict(
                type='str',
                required=True,
            ),
            managed_instance_name=dict(
                type='str',
                required=True,
            ),
            database_name=dict(
                type='str',
                required=True,
            ),
            collation=dict(
                type='str'
            ),
            location=dict(
                type='str'
            ),
            state=dict(
                type='str',
                choices=['present', 'absent'],
                default='present'
            ),
        )
        # store the results of the module operation
        self.results = dict(
            changed=False
        )
        self.resource_group = None
        self.managed_instance_name = None
        self.database_name = None
        self.state = None
        self.parameters = dict()

        super(AzureRMSqlMIDatabase, self).__init__(self.module_arg_spec, supports_check_mode=True, supports_tags=True, facts_module=False)

    def exec_module(self, **kwargs):
        for key in list(self.module_arg_spec.keys()) + ['tags']:
            if hasattr(self, key):
                setattr(self, key, kwargs[key])
            elif kwargs.get(key) is not None:
                self.parameters[key] = kwargs.get(key)

        changed = False
        resource_group = self.get_resource_group(self.resource_group)
        if self.parameters.get('location') is None:
            # Set default location
            self.parameters['location'] = resource_group.location

        old_response = self.get()
        if old_response is None:
            if self.state == 'present':
                changed = True
                if not self.check_mode:
                    self.results['database'] = self.create_database()
        else:
            update_tags, tags = self.update_tags(old_response.get('tags'))
            if update_tags:
                changed = True
                self.parameters['tags'] = tags
            for key in self.parameters.keys():
                if key != 'tags' and self.parameters[key] != old_response.get(key):
                    self.fail("The collection and location not support to update")
            if self.state == 'present':
                if changed and not self.check_mode:
                    self.results['database'] = self.update_database()
                else:
                    self.results['database'] = old_response
            else:
                changed = True
                if not self.check_mode:
                    self.results['database'] = self.delete_database()

        self.results['changed'] = changed
        return self.results

    def create_database(self):
        response = None
        try:
            response = self.sql_client.managed_databases.begin_create_or_update(resource_group_name=self.resource_group,
                                                                                managed_instance_name=self.managed_instance_name,
                                                                                database_name=self.database_name,
                                                                                parameters=self.parameters)
            self.log("Response : {0}".format(response))
        except HttpResponseError as ec:
            self.fail('Create the SQL managed instance database failed, exception as {0}'.format(ec))

        return self.format_item(self.get_poller_result(response))

    def update_database(self):
        response = None
        try:
            response = self.sql_client.managed_databases.begin_update(resource_group_name=self.resource_group,
                                                                      managed_instance_name=self.managed_instance_name,
                                                                      database_name=self.database_name,
                                                                      parameters=self.parameters)
            self.log("Response : {0}".format(response))
        except HttpResponseError as ec:
            self.fail('Update the SQL managed instance database failed, exception as {0}'.format(ec))

        return self.format_item(self.get_poller_result(response))

    def get(self):
        response = None
        try:
            response = self.sql_client.managed_databases.get(resource_group_name=self.resource_group,
                                                             managed_instance_name=self.managed_instance_name,
                                                             database_name=self.database_name)
            self.log("Response : {0}".format(response))
        except HttpResponseError as ec:
            self.log('Could not get facts for SQL managed instance database. Exception as {0}'.format(ec))

        return self.format_item(response)

    def delete_database(self):
        response = None
        try:
            response = self.sql_client.managed_databases.begin_delete(resource_group_name=self.resource_group,
                                                                      managed_instance_name=self.managed_instance_name,
                                                                      database_name=self.database_name)
            self.log("Response : {0}".format(response))
        except HttpResponseError as ec:
            self.fail('Could not get facts for SQL managed instance database. Exception as {0}'.format(ec))

        return self.format_item(self.get_poller_result(response))

    def format_item(self, item):
        if item is None:
            return
        d = item.as_dict()
        d = {
            'resource_group': self.resource_group,
            'managed_instance_name': self.managed_instance_name,
            'database_name': d.get('name'),
            'id': d.get('id', None),
            'type': d.get('type', None),
            'location': d.get('location'),
            'tags': d.get('tags'),
            'collation': d.get('collation'),
            'status': d.get('status'),
            'creation_date': d.get('creation_date'),
            'restore_point_in_time': d.get('restore_point_in_time'),
            'default_secondary_location': d.get('default_secondary_location'),
            'catalog_collation': d.get('catalog_collation'),
            'create_mode': d.get('create_mode'),
            'storage_container_uri': d.get('storage_container_uri'),
            'source_database_id': d.get('source_database_id'),
            'restorable_dropped_database_id': d.get('restorable_dropped_database_id'),
            'storage_container_sas_token': d.get('storage_container_sas_token'),
            'recoverable_database_id': d.get('recoverable_database_id'),
            'long_term_retention_backup_resource_id': d.get('long_term_retention_backup_resource_id'),
            'auto_complete_restore': d.get('auto_complete_restore'),
            'last_backup_name': d.get('last_backup_name')
        }
        return d


def main():
    AzureRMSqlMIDatabase()


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2023, Tim Cragg (@timcragg)
# Copyright: (c) 2023, Akini Ross (@akinross)
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {"metadata_version": "1.1", "status": ["preview"], "supported_by": "community"}

DOCUMENTATION = r"""
---
module: aci_vrf_multicast
short_description: Manage VRF Multicast objects (pim:CtxP)
description:
- Manage VRF Multicast objects on Cisco ACI fabrics.
- Creating I(state=present) enables Protocol Independent Multicast (PIM) on a VRF
- Deleting I(state=absent) disables Protocol Independent Multicast (PIM) on a VRF.
options:
  tenant:
    description:
    - The name of an existing tenant.
    type: str
    aliases: [ tenant_name ]
  vrf:
    description:
    - The name of an existing VRF.
    type: str
    aliases: [ vrf_name ]
  pim_setting:
    description: Configuration container for Protocol Independent Multicast (PIM) settings.
    type: dict
    suboptions:
      mtu:
        description:
        - The MTU size supported for multicast.
        - The APIC defaults to C(1500) when unset during creation.
        type: int
      control_state:
        description:
        - The action(s) to take when a loop is detected.
        - Specify C([]) to remove the control state configuration.
        type: list
        elements: str
        aliases: [ control, ctrl ]
        choices: [ fast, strict ]
  resource_policy:
    description: Configuration container for Protocol Independent Multicast (PIM) resource policy.
    type: dict
    suboptions:
      maximum_limit:
        description:
        - The Max Multicast Entries.
        - The APIC defaults to C(unlimited) when unset during creation.
        - Specify C(0) to reset to C(unlimited).
        type: int
        aliases: [ max ]
      reserved_multicast_entries:
        description:
        - The Reserved Multicast Entries.
        - The APIC defaults to C(undefined) when unset during creation.
        - Required when C(reserved_route_map) is provided.
        type: int
        aliases: [ rsvd ]
      reserved_route_map:
        description:
        - The DN of the Route Map.
        - Specify C("") to remove the Route Map configuration.
        - Required when C(reserved_multicast_entries) is provided.
        type: str
        aliases: [ route_map, route_map_dn ]
  any_source_multicast:
    description: Configuration container for Protocol Independent Multicast (PIM) Any Source Multicast (ASM) settings.
    type: dict
    aliases: [ asm, any_source ]
    suboptions:
      shared_range_route_map:
        description:
        - The DN of the Route Map.
        - Specify C("") to remove the Route Map configuration.
        type: str
        aliases: [ shared_range_policy ]
      source_group_expiry_route_map:
        description:
        - The DN of the Route Map.
        - Specify C("") to remove the Route Map configuration.
        type: str
        aliases: [ sg_expiry_route_map ]
      expiry:
        description:
        - The expiry time in seconds.
        - The APIC defaults to C(default-timeout) when unset during creation.
        - Specify C(0) to reset to C(default-timeout).
        type: int
        aliases: [ expiry_seconds ]
      max_rate:
        description:
        - The maximum rate per second.
        - The APIC defaults to C(65535) when unset during creation.
        type: int
        aliases: [ max_rate_per_second ]
      source_ip:
        description:
        - The source IP address.
        type: str
        aliases: [ source, source_ip_address ]
  source_specific_multicast:
    description: Configuration container for Protocol Independent Multicast (PIM) Source Specific Multicast (SSM) settings.
    type: dict
    aliases: [ ssm, specific_source ]
    suboptions:
      group_range_route_map:
        description:
        - The DN of the Route Map.
        - Specify C("") to remove the Route Map configuration.
        type: str
        aliases: [ group_range_policy ]
  bootstrap_router:
    description: Configuration container for Protocol Independent Multicast (PIM) Bootstrap Router (BSR) settings.
    type: dict
    aliases: [ bsr, bootstrap ]
    suboptions:
      bsr_filter:
        description:
        - The DN of the Route Map.
        - Specify C("") to remove the Route Map configuration.
        type: str
        aliases: [ filter, route_map, route_map_dn ]
      rp_updates:
        description:
        - The control state of the Bootstrap Router (BSR) policy.
        - Specify C([]) to remove the control state configuration.
        type: list
        elements: str
        aliases: [ control, ctrl ]
        choices: [ forward, listen ]
  auto_rp:
    description: Configuration container for Protocol Independent Multicast (PIM) Auto-Rendezvous Point (Auto-RP) settings.
    type: dict
    aliases: [ auto ]
    suboptions:
      ma_filter:
        description:
        - The DN of the Route Map.
        - Specify C("") to remove the Route Map configuration.
        type: str
        aliases: [ filter, route_map, route_map_dn ]
      rp_updates:
        description:
        - The control state of the Auto-Rendezvous Point (Auto-RP) policy.
        - Specify C([]) to remove the control state configuration.
        type: list
        elements: str
        aliases: [ control, ctrl ]
        choices: [ forward, listen ]
  state:
    description:
    - Use C(present) or C(absent) for adding or removing.
    - Use C(query) for listing an object or multiple objects.
    type: str
    choices: [ absent, present, query ]
    default: present
extends_documentation_fragment:
- cisco.aci.aci
- cisco.aci.annotation
- cisco.aci.owner

notes:
- The I(tenant) and I(vrf) must exist before using this module in your playbook.
  The M(cisco.aci.aci_tenant) and M(cisco.aci.aci_vrf) modules can be used for this.
seealso:
- name: APIC Management Information Model reference
  description: More information about the internal APIC class B(pim:CtxP).
  link: https://developer.cisco.com/docs/apic-mim-ref/
author:
- Tim Cragg (@timcragg)
- Akini Ross (@akinross)
"""

EXAMPLES = r"""
- name: Enable Multicast on a VRF
  cisco.aci.aci_vrf_multicast:
    host: apic
    username: admin
    password: SomeSecretePassword
    tenant: ansible_tenant
    vrf: ansible_vrf
    state: present
  delegate_to: localhost

- name: Change Multicast PIM Settings on a VRF
  cisco.aci.aci_vrf_multicast:
    host: apic
    username: admin
    password: SomeSecretePassword
    tenant: ansible_tenant
    vrf: ansible_vrf
    pim_setting:
      mtu: 2000
      control_state: [fast, strict]
    state: present
  delegate_to: localhost

- name: Change Multicast Resource Policy on a VRF
  cisco.aci.aci_vrf_multicast:
    host: apic
    username: admin
    password: SomeSecretePassword
    tenant: ansible_tenant
    vrf: ansible_vrf
    resource_policy:
      maximum_limit: 100
      reserved_multicast_entries: 20
      reserved_route_map: uni/tn-ansible_test/rtmap-ansible_test
    state: present
  delegate_to: localhost

- name: Remove Route-Map from Multicast Resource Policy on a VRF
  cisco.aci.aci_vrf_multicast:
    host: apic
    username: admin
    password: SomeSecretePassword
    tenant: ansible_tenant
    vrf: ansible_vrf
    resource_policy:
      reserved_route_map: ""
    state: present
  delegate_to: localhost

- name: Change Multicast Any Source Multicast (ASM) Settings on a VRF
  cisco.aci.aci_vrf_multicast:
    host: apic
    username: admin
    password: SomeSecretePassword
    tenant: ansible_tenant
    vrf: ansible_vrf
    any_source_multicast:
      shared_range_route_map: uni/tn-ansible_test/rtmap-ansible_test
      source_group_expiry_route_map: uni/tn-ansible_test/rtmap-ansible_test
      expiry: 500
      max_rate: 64000
      source_ip: 1.1.1.1
    state: present
  delegate_to: localhost

- name: Change Multicast Source Specific Multicast (SSM) Settings on a VRF
  cisco.aci.aci_vrf_multicast:
    host: apic
    username: admin
    password: SomeSecretePassword
    tenant: ansible_tenant
    vrf: ansible_vrf
    source_specific_multicast:
      group_range_route_map: uni/tn-ansible_test/rtmap-ansible_test
    state: present
  delegate_to: localhost

- name: Change Multicast Bootstrap Router (BSR) Settings on a VRF
  cisco.aci.aci_vrf_multicast:
    host: apic
    username: admin
    password: SomeSecretePassword
    tenant: ansible_tenant
    vrf: ansible_vrf
    bootstrap_router:
      bsr_filter: uni/tn-ansible_test/rtmap-ansible_test
      rp_updates: [forward, listen]
    state: present
  delegate_to: localhost

- name: Change Multicast Auto-Rendezvous Point (Auto-RP) Settings on a VRF
  cisco.aci.aci_vrf_multicast:
    host: apic
    username: admin
    password: SomeSecretePassword
    tenant: ansible_tenant
    vrf: ansible_vrf
    auto_rp:
      ma_filter: uni/tn-ansible_test/rtmap-ansible_test
      rp_updates: [forward, listen]
    state: present
  delegate_to: localhost

- name: Disable Multicast on a VRF
  cisco.aci.aci_vrf_multicast:
    host: apic
    username: admin
    password: SomeSecretePassword
    tenant: ansible_tenant
    vrf: ansible_vrf
    state: absent
  delegate_to: localhost

- name: Query Multicast Settings for a VRF
  cisco.aci.aci_vrf_multicast:
    host: apic
    username: admin
    password: SomeSecretePassword
    tenant: ansible_tenant
    vrf: ansible_vrf
    state: query
  delegate_to: localhost
  register: query_result

- name: Query Multicast Settings for all VRFs
  cisco.aci.aci_vrf_multicast:
    host: apic
    username: admin
    password: SomeSecretePassword
    state: query
  delegate_to: localhost
  register: query_result
"""

RETURN = r"""
   current:
     description: The existing configuration from the APIC after the module has finished
     returned: success
     type: list
     sample:
       [
           {
               "fvTenant": {
                   "attributes": {
                       "descr": "Production environment",
                       "dn": "uni/tn-production",
                       "name": "production",
                       "nameAlias": "",
                       "ownerKey": "",
                       "ownerTag": ""
                   }
               }
           }
       ]
   error:
     description: The error information as returned from the APIC
     returned: failure
     type: dict
     sample:
       {
           "code": "122",
           "text": "unknown managed object class foo"
       }
   raw:
     description: The raw output returned by the APIC REST API (xml or json)
     returned: parse error
     type: str
     sample: '<?xml version="1.0" encoding="UTF-8"?><imdata totalCount="1"><error code="122" text="unknown managed object class "/></imdata>'
   sent:
     description: The actual/minimal configuration pushed to the APIC
     returned: info
     type: list
     sample:
       {
           "fvTenant": {
               "attributes": {
                   "descr": "Production environment"
               }
           }
       }
   previous:
     description: The original configuration from the APIC before the module has started
     returned: info
     type: list
     sample:
       [
           {
               "fvTenant": {
                   "attributes": {
                       "descr": "Production",
                       "dn": "uni/tn-production",
                       "name": "production",
                       "nameAlias": "",
                       "ownerKey": "",
                       "ownerTag": ""
                   }
               }
           }
       ]
   proposed:
     description: The assembled configuration from the user-provided parameters
     returned: info
     type: dict
     sample:
       {
           "fvTenant": {
               "attributes": {
                   "descr": "Production environment",
                   "name": "production"
               }
           }
       }
   filter_string:
     description: The filter string used for the request
     returned: failure or debug
     type: str
     sample: ?rsp-prop-include=config-only
   method:
     description: The HTTP method used for the request to the APIC
     returned: failure or debug
     type: str
     sample: POST
   response:
     description: The HTTP response from the APIC
     returned: failure or debug
     type: str
     sample: OK (30 bytes)
   status:
     description: The HTTP status from the APIC
     returned: failure or debug
     type: int
     sample: 200
   url:
     description: The HTTP url used for the request to the APIC
     returned: failure or debug
     type: str
     sample: https://10.11.12.13/api/mo/uni/tn-production.json
   """

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.aci.plugins.module_utils.aci import ACIModule, aci_argument_spec, aci_annotation_spec, aci_owner_spec
from ansible_collections.cisco.aci.plugins.module_utils.constants import PIM_SETTING_CONTROL_STATE_MAPPING


def main():
    argument_spec = aci_argument_spec()
    argument_spec.update(aci_annotation_spec())
    argument_spec.update(aci_owner_spec())
    argument_spec.update(
        tenant=dict(type="str", aliases=["tenant_name"]),
        vrf=dict(type="str", aliases=["vrf_name"]),
        pim_setting=dict(
            type="dict",
            options=dict(
                mtu=dict(type="int"),
                control_state=dict(type="list", elements="str", choices=["fast", "strict"], aliases=["control", "ctrl"]),
            ),
        ),
        resource_policy=dict(
            type="dict",
            options=dict(
                maximum_limit=dict(type="int", aliases=["max"]),
                reserved_multicast_entries=dict(type="int", aliases=["rsvd"]),
                reserved_route_map=dict(type="str", aliases=["route_map", "route_map_dn"]),
            ),
        ),
        any_source_multicast=dict(
            type="dict",
            options=dict(
                shared_range_route_map=dict(type="str", aliases=["shared_range_policy"]),
                source_group_expiry_route_map=dict(type="str", aliases=["sg_expiry_route_map"]),
                expiry=(dict(type="int", aliases=["expiry_seconds"])),
                max_rate=(dict(type="int", aliases=["max_rate_per_second"])),
                source_ip=(dict(type="str", aliases=["source", "source_ip_address"])),
            ),
            aliases=["asm", "any_source"],
        ),
        source_specific_multicast=dict(
            type="dict",
            options=dict(
                group_range_route_map=dict(type="str", aliases=["group_range_policy"]),
            ),
            aliases=["ssm", "specific_source"],
        ),
        bootstrap_router=dict(
            type="dict",
            options=dict(
                bsr_filter=dict(type="str", aliases=["filter", "route_map", "route_map_dn"]),
                rp_updates=dict(type="list", elements="str", choices=["forward", "listen"], aliases=["control", "ctrl"]),
            ),
            aliases=["bsr", "bootstrap"],
        ),
        auto_rp=dict(
            type="dict",
            options=dict(
                ma_filter=dict(type="str", aliases=["filter", "route_map", "route_map_dn"]),
                rp_updates=dict(type="list", elements="str", choices=["forward", "listen"], aliases=["control", "ctrl"]),
            ),
            aliases=["auto"],
        ),
        state=dict(type="str", default="present", choices=["absent", "present", "query"]),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_if=[
            ["state", "absent", ["tenant", "vrf"]],
            ["state", "present", ["tenant", "vrf"]],
        ],
    )

    aci = ACIModule(module)

    tenant = module.params.get("tenant")
    vrf = module.params.get("vrf")
    pim_setting = module.params.get("pim_setting")
    resource_policy = module.params.get("resource_policy")
    any_source_multicast = module.params.get("any_source_multicast")
    source_specific_multicast = module.params.get("source_specific_multicast")
    bootstrap_router = module.params.get("bootstrap_router")
    auto_rp = module.params.get("auto_rp")
    state = module.params.get("state")

    if resource_policy and resource_policy.get("reserved_multicast_entries") and resource_policy.get("reserved_route_map") is None:
        aci.fail_json(msg="parameters are mutually exclusive: reserved_route_map|reserved_multicast_entries")
    elif resource_policy and resource_policy.get("reserved_route_map") and not resource_policy.get("reserved_multicast_entries"):
        aci.fail_json(msg="C(reserved_multicast_entries) must be provided and greater than 0 when C(reserved_route_map) is provided")

    aci.construct_url(
        root_class=dict(
            aci_class="fvTenant",
            aci_rn="tn-{0}".format(tenant),
            module_object=tenant,
            target_filter={"name": tenant},
        ),
        subclass_1=dict(
            aci_class="fvCtx",
            aci_rn="ctx-{0}".format(vrf),
            module_object=vrf,
            target_filter={"name": vrf},
        ),
        subclass_2=dict(
            aci_class="pimCtxP",
            aci_rn="pimctxp",
            target_filter={"name": ""},
        ),
        child_classes=["pimResPol", "pimASMPatPol", "pimSSMPatPol", "pimAutoRPPol", "pimBSRPPol"],
    )

    aci.get_existing()

    if state == "present":
        existing_config = aci.existing[0] if aci.existing else {}
        child_configs = []

        resource_policy_config = dict(pimResPol=dict(attributes=dict(name=""), children=[]))
        if resource_policy:
            max = "unlimited" if resource_policy.get("maximum_limit") == 0 else resource_policy.get("maximum_limit")
            if max is not None:
                resource_policy_config["pimResPol"]["attributes"]["max"] = str(max)

            reserved_route_map = resource_policy.get("reserved_route_map")
            if reserved_route_map is not None:
                existing_rrm = get_child_from_existing_config(existing_config, ["pimCtxP", "pimResPol", "rtdmcRsFilterToRtMapPol"])
                rsvd = resource_policy.get("reserved_multicast_entries")

                if (existing_rrm or reserved_route_map != "") and rsvd:
                    resource_policy_config["pimResPol"]["attributes"]["rsvd"] = str(rsvd)
                elif existing_rrm and reserved_route_map == "":
                    resource_policy_config["pimResPol"]["attributes"]["rsvd"] = "undefined"

                set_route_map_config(
                    existing_config,
                    resource_policy_config["pimResPol"]["children"],
                    ["pimCtxP", "pimResPol", "rtdmcRsFilterToRtMapPol"],
                    reserved_route_map,
                )

        child_configs.append(resource_policy_config)

        any_source_multicast_config = dict(
            pimASMPatPol=dict(
                attributes=dict(name=""),
                children=[
                    dict(pimSharedRangePol=dict(attributes=dict(name=""), children=[])),
                    dict(pimSGRangeExpPol=dict(attributes=dict(name=""), children=[])),
                    dict(pimRegTrPol=dict(attributes=dict(name=""), children=[])),
                ],
            )
        )
        if any_source_multicast:
            if any_source_multicast.get("shared_range_route_map") is not None:
                set_route_map_config(
                    existing_config,
                    any_source_multicast_config["pimASMPatPol"]["children"][0]["pimSharedRangePol"]["children"],
                    ["pimCtxP", "pimASMPatPol", "pimSharedRangePol", "rtdmcRsFilterToRtMapPol"],
                    any_source_multicast.get("shared_range_route_map"),
                )

            if any_source_multicast.get("source_group_expiry_route_map") is not None:
                set_route_map_config(
                    existing_config,
                    any_source_multicast_config["pimASMPatPol"]["children"][1]["pimSGRangeExpPol"]["children"],
                    ["pimCtxP", "pimASMPatPol", "pimSGRangeExpPol", "rtdmcRsFilterToRtMapPol"],
                    any_source_multicast.get("source_group_expiry_route_map"),
                )

            expiry = any_source_multicast.get("expiry")
            if expiry is not None:
                sg_expiry_config = "default-timeout" if any_source_multicast.get("expiry") == 0 else any_source_multicast.get("expiry")
                any_source_multicast_config["pimASMPatPol"]["children"][1]["pimSGRangeExpPol"]["attributes"]["sgExpItvl"] = str(sg_expiry_config)

            if any_source_multicast.get("max_rate") is not None:
                any_source_multicast_config["pimASMPatPol"]["children"][2]["pimRegTrPol"]["attributes"]["maxRate"] = str(any_source_multicast.get("max_rate"))

            if any_source_multicast.get("source_ip") is not None:
                any_source_multicast_config["pimASMPatPol"]["children"][2]["pimRegTrPol"]["attributes"]["srcIp"] = any_source_multicast.get("source_ip")

        child_configs.append(any_source_multicast_config)

        source_specific_multicast_config = dict(
            pimSSMPatPol=dict(
                attributes=dict(name=""),
                children=[
                    dict(pimSSMRangePol=dict(attributes=dict(name=""), children=[])),
                ],
            )
        )
        if source_specific_multicast and source_specific_multicast.get("group_range_route_map") is not None:
            set_route_map_config(
                existing_config,
                source_specific_multicast_config["pimSSMPatPol"]["children"][0]["pimSSMRangePol"]["children"],
                ["pimCtxP", "pimSSMPatPol", "pimSSMRangePol", "rtdmcRsFilterToRtMapPol"],
                source_specific_multicast.get("group_range_route_map"),
            )

        child_configs.append(source_specific_multicast_config)

        if bootstrap_router:
            bsr_config = dict(pimBSRPPol=dict(attributes=dict(name=""), children=[dict(pimBSRFilterPol=dict(attributes=dict(name=""), children=[]))]))
            if bootstrap_router.get("bsr_filter") is not None:
                set_route_map_config(
                    existing_config,
                    bsr_config["pimBSRPPol"]["children"][0]["pimBSRFilterPol"]["children"],
                    ["pimCtxP", "pimBSRPPol", "pimBSRFilterPol", "rtdmcRsFilterToRtMapPol"],
                    bootstrap_router.get("bsr_filter"),
                )

            rp_updates = bootstrap_router.get("rp_updates")
            if rp_updates is not None:
                bsr_config["pimBSRPPol"]["attributes"]["ctrl"] = ",".join(sorted(rp_updates))

            child_configs.append(bsr_config)

        if auto_rp:
            auto_rp_config = dict(pimAutoRPPol=dict(attributes=dict(name=""), children=[dict(pimMAFilterPol=dict(attributes=dict(name=""), children=[]))]))

            if auto_rp.get("ma_filter") is not None:
                set_route_map_config(
                    existing_config,
                    auto_rp_config["pimAutoRPPol"]["children"][0]["pimMAFilterPol"]["children"],
                    ["pimCtxP", "pimAutoRPPol", "pimMAFilterPol", "rtdmcRsFilterToRtMapPol"],
                    auto_rp.get("ma_filter"),
                )

            rp_updates = auto_rp.get("rp_updates")
            if rp_updates is not None:
                auto_rp_config["pimAutoRPPol"]["attributes"]["ctrl"] = ",".join(sorted(rp_updates))

            child_configs.append(auto_rp_config)

        mtu = None
        control_state = None
        if pim_setting:
            mtu = pim_setting.get("mtu")
            control_state = (
                ",".join(sorted([PIM_SETTING_CONTROL_STATE_MAPPING.get(v) for v in pim_setting.get("control_state")]))
                if pim_setting.get("control_state") is not None
                else None
            )

        aci.payload(
            aci_class="pimCtxP",
            class_config=dict(mtu=mtu, ctrl=control_state),
            child_configs=child_configs,
        )

        aci.get_diff(aci_class="pimCtxP")

        aci.post_config()

    elif state == "absent":
        aci.delete_config()

    aci.exit_json()


def get_child_from_existing_config(config, class_names):
    parent = class_names[0]
    class_names.remove(parent)

    for child in config.get(parent, {}).get("children", []):
        if len(class_names) == 1 and class_names[0] in child.keys():
            return child
        elif child.get(class_names[0], {}).get("children"):
            return get_child_from_existing_config(child, class_names)


def set_route_map_config(existing_config, new_config, class_names, route_map):
    existing_route_map = get_child_from_existing_config(existing_config, class_names)
    if route_map == "" and existing_route_map:
        new_config.append(dict(rtdmcRsFilterToRtMapPol=dict(attributes=dict(tDn=route_map, status="deleted"))))
    elif route_map:
        new_config.append(dict(rtdmcRsFilterToRtMapPol=dict(attributes=dict(tDn=route_map))))


if __name__ == "__main__":
    main()



from __future__ import absolute_import, division, print_function

__metaclass__ = type

import pytest
import random

from awx.main.models import Organization, Credential, CredentialType


@pytest.mark.django_db
def test_create_organization(run_module, admin_user):

    module_args = {
        'name': 'foo',
        'description': 'barfoo',
        'state': 'present',
        'max_hosts': '0',
        'controller_host': None,
        'controller_username': None,
        'controller_password': None,
        'validate_certs': None,
        'controller_oauthtoken': None,
        'controller_config_file': None,
    }

    result = run_module('organization', module_args, admin_user)
    assert result.get('changed'), result

    org = Organization.objects.get(name='foo')
    assert result == {"name": "foo", "changed": True, "id": org.id, "invocation": {"module_args": module_args}}

    assert org.description == 'barfoo'


@pytest.mark.django_db
def test_galaxy_credential_order(run_module, admin_user):
    org = Organization.objects.create(name='foo')
    cred_type = CredentialType.defaults['galaxy_api_token']()
    cred_type.save()

    cred_ids = []
    for number in range(1, 10):
        new_cred = Credential.objects.create(name=f"Galaxy Credential {number}", credential_type=cred_type, organization=org, inputs={'url': 'www.redhat.com'})
        cred_ids.append(new_cred.id)

    random.shuffle(cred_ids)

    module_args = {
        'name': 'foo',
        'state': 'present',
        'controller_host': None,
        'controller_username': None,
        'controller_password': None,
        'validate_certs': None,
        'controller_oauthtoken': None,
        'controller_config_file': None,
        'galaxy_credentials': cred_ids,
    }

    result = run_module('organization', module_args, admin_user)
    print(result)
    assert result['changed'] is True

    cred_order_in_org = []
    for a_cred in org.galaxy_credentials.all():
        cred_order_in_org.append(a_cred.id)

    assert cred_order_in_org == cred_ids

    # Shuffle them up and try again to make sure a new order is honored
    random.shuffle(cred_ids)

    module_args = {
        'name': 'foo',
        'state': 'present',
        'controller_host': None,
        'controller_username': None,
        'controller_password': None,
        'validate_certs': None,
        'controller_oauthtoken': None,
        'controller_config_file': None,
        'galaxy_credentials': cred_ids,
    }

    result = run_module('organization', module_args, admin_user)
    assert result['changed'] is True

    cred_order_in_org = []
    for a_cred in org.galaxy_credentials.all():
        cred_order_in_org.append(a_cred.id)

    assert cred_order_in_org == cred_ids



#!/usr/bin/python
# -*- coding: utf-8 -*-

# (c) 2018, Ansible by Red Hat, inc
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import absolute_import, division, print_function


__metaclass__ = type


DOCUMENTATION = """
module: netconf_get
author:
  - Ganesh Nalawade (@ganeshrn)
  - Sven Wisotzky (@wisotzky)
short_description: Fetch configuration/state data from NETCONF enabled network devices.
description:
  - NETCONF is a network management protocol developed and standardized by the IETF.
    It is documented in RFC 6241.
  - This module allows the user to fetch configuration and state data from NETCONF enabled
    network devices.
version_added: 1.0.0
extends_documentation_fragment:
  - ansible.netcommon.network_agnostic
options:
  source:
    description:
      - This argument specifies the datastore from which configuration data should be
        fetched. Valid values are I(running), I(candidate) and I(startup). If the C(source)
        value is not set both configuration and state information are returned in response
        from running datastore.
    type: str
    choices:
      - running
      - candidate
      - startup
  filter:
    description:
      - This argument specifies the string which acts as a filter to restrict the
        portions of the data to be are retrieved from the remote device. If this option
        is not specified entire configuration or state data is returned in result depending
        on the value of C(source) option. The C(filter) value can be either XML string
        or XPath or JSON string or native python dictionary, if the filter is in XPath
        format the NETCONF server running on remote host should support xpath capability
        else it will result in an error. If the filter is in JSON format the xmltodict library
        should be installed on the control node for JSON to XML conversion.
    type: raw
  display:
    description:
      - Encoding scheme to use when serializing output from the device. The option I(json)
        will serialize the output as JSON data. If the option value is I(json) it requires
        jxmlease to be installed on control node. The option I(pretty) is similar to
        received XML response but is using human readable format (spaces, new lines).
        The option value I(xml) is similar to received XML response but removes all
        XML namespaces.
    type: str
    choices:
      - json
      - pretty
      - xml
      - native
  lock:
    description:
      - Instructs the module to explicitly lock the datastore specified as C(source).
        If no I(source) is defined, the I(running) datastore will be locked. By setting
        the option value I(always) is will explicitly lock the datastore mentioned in
        C(source) option. By setting the option value I(never) it will not lock the
        C(source) datastore. The value I(if-supported) allows better interworking with
        NETCONF servers, which do not support the (un)lock operation for all supported
        datastores.
    type: str
    default: never
    choices:
      - never
      - always
      - if-supported
requirements:
  - ncclient (>=v0.5.2)
  - jxmlease (for display=json)
  - xmltodict (for display=native)
notes:
  - This module requires the NETCONF system service be enabled on the remote device
    being managed.
  - This module supports the use of connection=netconf
"""

EXAMPLES = """
- name: Get running configuration and state data
  ansible.netcommon.netconf_get:

- name: Get configuration and state data from startup datastore
  ansible.netcommon.netconf_get:
    source: startup

- name: Get system configuration data from running datastore state (junos)
  ansible.netcommon.netconf_get:
    source: running
    filter: <configuration><system></system></configuration>

- name: Get configuration and state data in JSON format
  ansible.netcommon.netconf_get:
    display: json

- name: get schema list using subtree w/ namespaces
  ansible.netcommon.netconf_get:
    display: json
    filter: <netconf-state xmlns="urn:ietf:params:xml:ns:yang:ietf-netconf-monitoring"><schemas><schema/></schemas></netconf-state>
    lock: never

- name: get schema list using xpath
  ansible.netcommon.netconf_get:
    display: xml
    filter: /netconf-state/schemas/schema

- name: get interface configuration with filter (iosxr)
  ansible.netcommon.netconf_get:
    display: pretty
    filter: <interface-configurations xmlns="http://cisco.com/ns/yang/Cisco-IOS-XR-ifmgr-cfg"></interface-configurations>
    lock: if-supported

- name: Get system configuration data from running datastore state (junos)
  ansible.netcommon.netconf_get:
    source: running
    filter: <configuration><system></system></configuration>
    lock: if-supported

- name: Get complete configuration data from running datastore (SROS)
  ansible.netcommon.netconf_get:
    source: running
    filter: <configure xmlns="urn:nokia.com:sros:ns:yang:sr:conf"/>

- name: Get complete state data (SROS)
  ansible.netcommon.netconf_get:
    filter: <state xmlns="urn:nokia.com:sros:ns:yang:sr:state"/>

- name: "get configuration with json filter string and native output (using xmltodict)"
  netconf_get:
    filter: |
      {
          "interface-configurations": {
              "@xmlns": "http://cisco.com/ns/yang/Cisco-IOS-XR-ifmgr-cfg",
              "interface-configuration": null
          }
      }
    display: native

- name: Define the Cisco IOSXR interface filter
  set_fact:
    filter:
      interface-configurations:
        "@xmlns": "http://cisco.com/ns/yang/Cisco-IOS-XR-ifmgr-cfg"
        interface-configuration: null

- name: "get configuration with native filter type using set_facts"
  ansible.netcommon.netconf_get:
    filter: "{{ filter }}"
    display: native
  register: result

- name: "get configuration with direct native filter type"
  ansible.netcommon.netconf_get:
    filter:
      {
        "interface-configurations":
          {
            "@xmlns": "http://cisco.com/ns/yang/Cisco-IOS-XR-ifmgr-cfg",
            "interface-configuration": null,
          },
      }
    display: native
  register: result

# Make a round-trip interface description change, diff the before and after
# this demonstrates the use of the native display format and several utilities
# from the ansible.utils collection

- name: Define the openconfig interface filter
  set_fact:
    filter:
      interfaces:
        "@xmlns": "http://openconfig.net/yang/interfaces"
        interface:
          name: Ethernet2

- name: Get the pre-change config using the filter
  ansible.netcommon.netconf_get:
    source: running
    filter: "{{ filter }}"
    display: native
  register: pre

- name: Update the description
  ansible.utils.update_fact:
    updates:
      - path: pre.output.data.interfaces.interface.config.description
        value: "Configured by ansible {{ 100 | random }}"
  register: updated

- name: Apply the new configuration
  ansible.netcommon.netconf_config:
    content:
      config:
        interfaces: "{{ updated.pre.output.data.interfaces }}"

- name: Get the post-change config using the filter
  ansible.netcommon.netconf_get:
    source: running
    filter: "{{ filter }}"
    display: native
  register: post

- name: Show the differences between the pre and post configurations
  ansible.utils.fact_diff:
    before: "{{ pre.output.data|ansible.utils.to_paths }}"
    after: "{{ post.output.data|ansible.utils.to_paths }}"
# TASK [Show the differences between the pre and post configurations] ********
# --- before
# +++ after
# @@ -1,11 +1,11 @@
#  {
# -    "@time-modified": "2020-10-23T12:27:17.462332477Z",
# +    "@time-modified": "2020-10-23T12:27:21.744541708Z",
#      "@xmlns": "urn:ietf:params:xml:ns:netconf:base:1.0",
#      "interfaces.interface.aggregation.config['fallback-timeout']['#text']": "90",
#      "interfaces.interface.aggregation.config['fallback-timeout']['@xmlns']": "http://arista.com/yang/openconfig/interfaces/augments",
#      "interfaces.interface.aggregation.config['min-links']": "0",
#      "interfaces.interface.aggregation['@xmlns']": "http://openconfig.net/yang/interfaces/aggregate",
# -    "interfaces.interface.config.description": "Configured by ansible 56",
# +    "interfaces.interface.config.description": "Configured by ansible 67",
#      "interfaces.interface.config.enabled": "true",
#      "interfaces.interface.config.mtu": "0",
#      "interfaces.interface.config.name": "Ethernet2",
"""

RETURN = """
stdout:
  description: The raw XML string containing configuration or state data
    received from the underlying ncclient library.
  returned: always apart from low-level errors (such as action plugin)
  type: str
  sample: "..."
stdout_lines:
  description: The value of stdout split into a list
  returned: always apart from low-level errors (such as action plugin)
  type: list
  sample: ["...", "..."]
output:
  description:
    Based on the value of display option will return either the set of
    transformed XML to JSON format from the RPC response with type dict
    or pretty XML string response (human-readable) or response with
    namespace removed from XML string.
  returned:
    If the display format is selected as I(json) it is returned as dict type
    and the conversion is done using jxmlease python library. If the display
    format is selected as I(native) it is returned as dict type and the conversion
    is done using xmltodict python library. If the display format is xml or pretty
    it is returned as a string apart from low-level errors (such as action plugin).
  type: complex
  contains:
    formatted_output:
      description:
        - Contains formatted response received from remote host as per the value in display format.
      type: str
"""
try:
    from lxml.etree import tostring
except ImportError:
    from xml.etree.ElementTree import tostring

from ansible.module_utils._text import to_text
from ansible.module_utils.basic import AnsibleModule

from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.netconf import (
    remove_namespaces,
)
from ansible_collections.ansible.netcommon.plugins.module_utils.network.netconf.netconf import (
    get,
    get_capabilities,
    get_config,
)
from ansible_collections.ansible.netcommon.plugins.module_utils.utils.data import (
    dict_to_xml,
    validate_and_normalize_data,
    xml_to_dict,
)


try:
    import jxmlease

    HAS_JXMLEASE = True
except ImportError:
    HAS_JXMLEASE = False


def main():
    """entry point for module execution"""
    argument_spec = dict(
        source=dict(choices=["running", "candidate", "startup"]),
        filter=dict(type="raw"),
        display=dict(choices=["json", "pretty", "xml", "native"]),
        lock=dict(default="never", choices=["never", "always", "if-supported"]),
    )

    module = AnsibleModule(argument_spec=argument_spec, supports_check_mode=True)

    capabilities = get_capabilities(module)
    operations = capabilities["device_operations"]

    source = module.params["source"]
    filter = module.params["filter"]

    try:
        filter_data, filter_type = validate_and_normalize_data(filter)
    except Exception as exc:
        module.fail_json(msg=to_text(exc))

    if filter_type == "xml":
        filter_type = "subtree"
    elif filter_type == "json":
        try:
            filter = dict_to_xml(filter_data)
        except Exception as exc:
            module.fail_json(msg=to_text(exc))
        filter_type = "subtree"
    elif filter_type == "xpath":
        pass
    elif filter_type is None:
        if filter_data is not None:
            # to maintain backward compatibility for ansible 2.9 which
            # defaults to "subtree" filter type
            filter_type = "subtree"
            module.warn(
                "The data format of filter option value couldn't be identified, hence set to 'subtree'"
            )
        else:
            pass
    elif filter_type:
        module.fail_json(
            msg="Invalid filter type detected %s for filter value %s" % (filter_type, filter)
        )

    lock = module.params["lock"]
    display = module.params["display"]

    if source == "candidate" and not operations.get("supports_commit", False):
        module.fail_json(msg="candidate source is not supported on this device")

    if source == "startup" and not operations.get("supports_startup", False):
        module.fail_json(msg="startup source is not supported on this device")

    if filter_type == "xpath" and not operations.get("supports_xpath", False):
        module.fail_json(
            msg="filter value '%s' of type xpath is not supported on this device" % filter
        )

    # If source is None, NETCONF <get> operation is issued, reading config/state data
    # from the running datastore. The python expression "(source or 'running')" results
    # in the value of source (if not None) or the value 'running' (if source is None).

    if lock == "never":
        execute_lock = False
    elif (source or "running") in operations.get("lock_datastore", []):
        # lock is requested (always/if-support) and supported => lets do it
        execute_lock = True
    else:
        # lock is requested (always/if-supported) but not supported => issue warning
        module.warn(
            "lock operation on '%s' source is not supported on this device" % (source or "running")
        )
        execute_lock = lock == "always"

    if display == "json" and not HAS_JXMLEASE:
        module.fail_json(
            msg="jxmlease is required to display response in json format"
            "but does not appear to be installed. "
            "It can be installed using `pip install jxmlease`"
        )

    filter_spec = (filter_type, filter) if filter_type else None

    if source is not None:
        response = get_config(module, source, filter_spec, execute_lock)
    else:
        response = get(module, filter_spec, execute_lock)

    xml_resp = to_text(tostring(response))
    output = None

    if display == "xml":
        output = remove_namespaces(xml_resp)
    elif display == "json":
        try:
            output = jxmlease.parse(xml_resp)
        except Exception:
            raise ValueError(xml_resp)
    elif display == "pretty":
        output = to_text(tostring(response, pretty_print=True))
    elif display == "native":
        try:
            output = xml_to_dict(xml_resp)
        except Exception as exc:
            module.fail_json(msg=to_text(exc))

    result = {"stdout": xml_resp, "output": output}

    module.exit_json(**result)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# coding: utf-8 -*-

# (c) 2017, Wayne Witzel III <wayne@riotousliving.com>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type


ANSIBLE_METADATA = {'metadata_version': '1.1', 'status': ['preview'], 'supported_by': 'community'}


DOCUMENTATION = '''
---
module: role
author: "Wayne Witzel III (@wwitzel3)"
short_description: grant or revoke an Automation Platform Controller role.
description:
    - Roles are used for access control, this module is for managing user access to server resources.
    - Grant or revoke Automation Platform Controller roles to users. See U(https://www.ansible.com/tower) for an overview.
options:
    user:
      description:
        - User name, ID, or named URL that receives the permissions specified by the role.
        - Deprecated, use 'users'.
      type: str
    users:
      description:
        - User names, IDs, or named URLs that receive the permissions specified by the role.
      type: list
      elements: str
    team:
      description:
        - Team name, ID, or named URL that receives the permissions specified by the role.
        - Deprecated, use 'teams'.
      type: str
    teams:
      description:
        - Team names, IDs, or named URLs that receive the permissions specified by the role.
      type: list
      elements: str
    role:
      description:
        - The role type to grant/revoke.
      required: True
      choices: ["admin", "read", "member", "execute", "adhoc", "update", "use", "approval", "auditor", "project_admin", "inventory_admin", "credential_admin",
                "workflow_admin", "notification_admin", "job_template_admin", "execution_environment_admin"]
      type: str
    target_team:
      description:
        - Team name, ID, or named URL that the role acts on.
        - For example, make someone a member or an admin of a team.
        - Members of a team implicitly receive the permissions that the team has.
        - Deprecated, use 'target_teams'.
      type: str
    target_teams:
      description:
        - Team names, IDs, or named URLs that the role acts on.
        - For example, make someone a member or an admin of a team.
        - Members of a team implicitly receive the permissions that the team has.
      type: list
      elements: str
    inventory:
      description:
        - Inventory name, ID, or named URL the role acts on.
        - Deprecated, use 'inventories'.
      type: str
    inventories:
      description:
        - Inventory names, IDs, or named URLs the role acts on.
      type: list
      elements: str
    job_template:
      description:
        - The job template name, ID, or named URL the role acts on.
        - Deprecated, use 'job_templates'.
      type: str
    job_templates:
      description:
        - The job template names, IDs, or named URLs the role acts on.
      type: list
      elements: str
    workflow:
      description:
        - The workflow job template name, ID, or named URL the role acts on.
        - Deprecated, use 'workflows'.
      type: str
    workflows:
      description:
        - The workflow job template names, IDs, or named URLs the role acts on.
      type: list
      elements: str
    credential:
      description:
        - Credential name, ID, or named URL the role acts on.
        - Deprecated, use 'credentials'.
      type: str
    credentials:
      description:
        - Credential names, IDs, or named URLs the role acts on.
      type: list
      elements: str
    organization:
      description:
        - Organization name, ID, or named URL the role acts on.
        - Deprecated, use 'organizations'.
      type: str
    organizations:
      description:
        - Organization names, IDs, or named URLs the role acts on.
      type: list
      elements: str
    lookup_organization:
      description:
        - Organization name, ID, or named URL the inventories, job templates, projects, or workflows the items exists in.
        - Used to help lookup the object, for organization roles see organization.
        - If not provided, will lookup by name only, which does not work with duplicates.
      type: str
    project:
      description:
        - Project name, ID, or named URL the role acts on.
        - Deprecated, use 'projects'.
      type: str
    projects:
      description:
        - Project names, IDs, or named URLs the role acts on.
      type: list
      elements: str
    instance_groups:
      description:
        - Instance Group names, IDs, or named URLs the role acts on.
      type: list
      elements: str
    state:
      description:
        - Desired state.
        - State of present indicates the user should have the role.
        - State of absent indicates the user should have the role taken away, if they have it.
      default: "present"
      choices: ["present", "absent"]
      type: str

extends_documentation_fragment: awx.awx.auth
'''


EXAMPLES = '''
- name: Add jdoe to the member role of My Team
  role:
    user: jdoe
    target_team: "My Team"
    role: member
    state: present

- name: Add Joe to multiple job templates and a workflow
  role:
    user: joe
    role: execute
    workflows:
      - test-role-workflow
    job_templates:
      - jt1
      - jt2
    state: present
'''

from ..module_utils.controller_api import ControllerAPIModule


def main():

    argument_spec = dict(
        user=dict(),
        users=dict(type='list', elements='str'),
        team=dict(),
        teams=dict(type='list', elements='str'),
        role=dict(
            choices=[
                "admin",
                "read",
                "member",
                "execute",
                "adhoc",
                "update",
                "use",
                "approval",
                "auditor",
                "project_admin",
                "inventory_admin",
                "credential_admin",
                "workflow_admin",
                "notification_admin",
                "job_template_admin",
                "execution_environment_admin",
            ],
            required=True,
        ),
        target_team=dict(),
        target_teams=dict(type='list', elements='str'),
        inventory=dict(),
        inventories=dict(type='list', elements='str'),
        job_template=dict(),
        job_templates=dict(type='list', elements='str'),
        workflow=dict(),
        workflows=dict(type='list', elements='str'),
        credential=dict(),
        credentials=dict(type='list', elements='str'),
        organization=dict(),
        organizations=dict(type='list', elements='str'),
        lookup_organization=dict(),
        project=dict(),
        projects=dict(type='list', elements='str'),
        instance_groups=dict(type='list', elements='str'),
        state=dict(choices=['present', 'absent'], default='present'),
    )

    module = ControllerAPIModule(argument_spec=argument_spec)

    role_type = module.params.pop('role')
    role_field = role_type + '_role'
    state = module.params.pop('state')

    module.json_output['role'] = role_type

    # Deal with legacy parameters
    resource_list_param_keys = {
        'credentials': 'credential',
        'inventories': 'inventory',
        'job_templates': 'job_template',
        'organizations': 'organization',
        'projects': 'project',
        'target_teams': 'target_team',
        'workflows': 'workflow',
        'users': 'user',
        'teams': 'team',
        'instance_groups': 'instance_group',
    }

    resources = {}
    for resource_group, old_name in resource_list_param_keys.items():
        if module.params.get(resource_group) is not None:
            resources.setdefault(resource_group, []).extend(module.params.get(resource_group))
        if module.params.get(old_name) is not None:
            resources.setdefault(resource_group, []).append(module.params.get(old_name))
    if module.params.get('lookup_organization') is not None:
        resources['lookup_organization'] = module.params.get('lookup_organization')
    if module.params.get('instance_groups') is not None:
        resources['instance_groups'] = module.params.get('instance_groups')
    # Change workflows to its endpoint name.
    if 'workflows' in resources:
        resources['workflow_job_templates'] = resources.pop('workflows')

    # Set lookup data to use
    lookup_data = {}
    if 'lookup_organization' in resources:
        lookup_data['organization'] = module.resolve_name_to_id('organizations', resources['lookup_organization'])
        resources.pop('lookup_organization')

    # Lookup actor data
    # separate actors from resources
    actor_data = {}
    missing_items = []
    # Lookup Resources
    resource_data = {}
    for key, value in resources.items():
        for resource in value:
            # Attempt to look up project based on the provided name, ID, or named URL and lookup data
            lookup_key = key
            if key == 'organizations' or key == 'users':
                lookup_data_populated = {}
            else:
                lookup_data_populated = lookup_data
            if key == 'target_teams':
                lookup_key = 'teams'
            data = module.get_one(lookup_key, name_or_id=resource, data=lookup_data_populated)
            if data is None:
                missing_items.append(resource)
            else:
                if key == 'users' or key == 'teams':
                    actor_data.setdefault(key, []).append(data)
                elif key == 'target_teams':
                    resource_data.setdefault('teams', []).append(data)
                else:
                    resource_data.setdefault(key, []).append(data)
    if len(missing_items) > 0:
        module.fail_json(
            msg='There were {0} missing items, missing items: {1}'.format(len(missing_items), missing_items), changed=False
        )

    # build association agenda
    associations = {}
    for actor_type, actors in actor_data.items():
        for key, value in resource_data.items():
            for resource in value:
                resource_roles = resource['summary_fields']['object_roles']
                if role_field not in resource_roles:
                    available_roles = ', '.join(list(resource_roles.keys()))
                    module.fail_json(
                        msg='Resource {0} has no role {1}, available roles: {2}'.format(resource['url'], role_field, available_roles), changed=False
                    )
                role_data = resource_roles[role_field]
                endpoint = '/roles/{0}/{1}/'.format(role_data['id'], actor_type)
                associations.setdefault(endpoint, [])
                for actor in actors:
                    associations[endpoint].append(actor['id'])

    # perform associations
    for association_endpoint, new_association_list in associations.items():
        response = module.get_all_endpoint(association_endpoint)
        existing_associated_ids = [association['id'] for association in response['json']['results']]

        if state == 'present':
            for an_id in list(set(new_association_list) - set(existing_associated_ids)):
                response = module.post_endpoint(association_endpoint, **{'data': {'id': int(an_id)}})
                if response['status_code'] == 204:
                    module.json_output['changed'] = True
                else:
                    module.fail_json(msg="Failed to grant role. {0}".format(response['json'].get('detail', response['json'].get('msg', 'unknown'))))
        else:
            for an_id in list(set(existing_associated_ids) & set(new_association_list)):
                response = module.post_endpoint(association_endpoint, **{'data': {'id': int(an_id), 'disassociate': True}})
                if response['status_code'] == 204:
                    module.json_output['changed'] = True
                else:
                    module.fail_json(msg="Failed to revoke role. {0}".format(response['json'].get('detail', response['json'].get('msg', 'unknown'))))

    module.exit_json(**module.json_output)


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2020 Jose Angel Munoz, <josea.munoz@gmail.com>
#
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
#

from __future__ import absolute_import, division, print_function
__metaclass__ = type

DOCUMENTATION = '''
---
module: azure_rm_privatednszone

version_added: "0.0.1"

short_description: Manage Azure private DNS zones

description:
    - Creates and deletes Azure private DNS zones.

options:
    resource_group:
        description:
            - Name of resource group.
        type: str
        required: true
    name:
        description:
            - Name of the private DNS zone.
        type: str
        required: true
    state:
        description:
            - Assert the state of the zone. Use C(present) to create or update and C(absent) to delete.
        default: present
        type: str
        choices:
            - absent
            - present

extends_documentation_fragment:
    - azure.azcollection.azure
    - azure.azcollection.azure_tags

author:
    - Jose Angel Munoz (@imjoseangel)
'''

EXAMPLES = '''

- name: Create a private DNS zone
  azure_rm_privatednszone:
    resource_group: myResourceGroup
    name: example.com

- name: Delete a private DNS zone
  azure_rm_privatednszone:
    resource_group: myResourceGroup
    name: example.com
    state: absent
'''

RETURN = '''
state:
    description:
        - Current state of the zone.
    returned: always
    type: dict
    sample: {
        "id": "/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroup/myResourceGroup",
        "location": "global",
        "name": "Testing",
        "number_of_record_sets": 2,
        "number_of_virtual_network_links": 0,
        "number_of_virtual_network_links_with_registration": 0
    }

'''

from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase

try:
    from azure.core.exceptions import ResourceNotFoundError
    from azure.core.polling import LROPoller
except ImportError:
    # This is handled in azure_rm_common
    pass


class AzureRMPrivateDNSZone(AzureRMModuleBase):
    def __init__(self):

        # define user inputs from playbook
        self.module_arg_spec = dict(resource_group=dict(type='str',
                                                        required=True),
                                    name=dict(type='str', required=True),
                                    state=dict(choices=['present', 'absent'],
                                               default='present',
                                               type='str'))

        # store the results of the module operation
        self.results = dict(changed=False, state=dict())

        self.resource_group = None
        self.name = None
        self.state = None
        self.tags = None

        super(AzureRMPrivateDNSZone, self).__init__(self.module_arg_spec,
                                                    supports_check_mode=True,
                                                    supports_tags=True)

    def exec_module(self, **kwargs):

        # create a new zone variable in case the 'try' doesn't find a zone
        zone = None
        for key in list(self.module_arg_spec.keys()) + ['tags']:
            setattr(self, key, kwargs[key])

        self.results['check_mode'] = self.check_mode

        # retrieve resource group to make sure it exists
        self.get_resource_group(self.resource_group)

        changed = False
        results = dict()

        try:
            self.log('Fetching private DNS zone {0}'.format(self.name))
            zone = self.private_dns_client.private_zones.get(
                self.resource_group, self.name)

            # serialize object into a dictionary
            results = zone_to_dict(zone)

            # don't change anything if creating an existing zone, but change if deleting it
            if self.state == 'present':
                changed = False

                update_tags, results['tags'] = self.update_tags(
                    results['tags'])
                if update_tags:
                    changed = True
            elif self.state == 'absent':
                changed = True

        except ResourceNotFoundError:
            # the zone does not exist so create it
            if self.state == 'present':
                changed = True
            else:
                # you can't delete what is not there
                changed = False

        self.results['changed'] = changed
        self.results['state'] = results

        # return the results if your only gathering information
        if self.check_mode:
            return self.results

        if changed:
            if self.state == 'present':
                zone = self.private_dns_models.PrivateZone(tags=self.tags,
                                                           location='global')
                self.results['state'] = self.create_or_update_zone(zone)
            elif self.state == 'absent':
                # delete zone
                self.delete_zone()
                # the delete does not actually return anything. if no exception, then we'll assume
                # it worked.
                self.results['state']['status'] = 'Deleted'

        return self.results

    def create_or_update_zone(self, zone):
        try:
            # create or update the new Zone object we created
            new_zone = self.private_dns_client.private_zones.begin_create_or_update(
                self.resource_group, self.name, zone)

            if isinstance(new_zone, LROPoller):
                new_zone = self.get_poller_result(new_zone)

        except Exception as exc:
            self.fail("Error creating or updating zone {0} - {1}".format(
                self.name, exc.message or str(exc)))
        return zone_to_dict(new_zone)

    def delete_zone(self):
        try:
            # delete the Zone
            poller = self.private_dns_client.private_zones.begin_delete(
                self.resource_group, self.name)
            result = self.get_poller_result(poller)
        except Exception as exc:
            self.fail("Error deleting zone {0} - {1}".format(
                self.name, exc.message or str(exc)))
        return result


def zone_to_dict(zone):
    # turn Zone object into a dictionary (serialization)
    result = dict(
        id=zone.id,
        name=zone.name,
        number_of_record_sets=zone.number_of_record_sets,
        number_of_virtual_network_links=zone.number_of_virtual_network_links,
        number_of_virtual_network_links_with_registration=zone.
        number_of_virtual_network_links_with_registration,
        tags=zone.tags)
    return result


def main():
    AzureRMPrivateDNSZone()


if __name__ == '__main__':
    main()
